@@ -4,6 +4,7 @@ import (
 	"bytes"
 	"errors"
 	"fmt"
+	"strings"
 	"sync"
 	"time"
 
@@ -12,6 +13,7 @@ import (
 	"git.sr.ht/~rjarry/aerc/lib/hooks"
 	"git.sr.ht/~rjarry/aerc/lib/log"
 	"git.sr.ht/~rjarry/aerc/lib/marker"
+	"git.sr.ht/~rjarry/aerc/lib/pama"
 	"git.sr.ht/~rjarry/aerc/lib/sort"
 	"git.sr.ht/~rjarry/aerc/lib/state"
 	"git.sr.ht/~rjarry/aerc/lib/templates"
@@ -35,7 +37,6 @@ type AccountView struct {
 	worker  *types.Worker
 	state   state.AccountState
 	newConn bool // True if this is a first run after a new connection/reconnection
-	uiConf  *config.UIConfig
 
 	split         *MessageViewer
 	splitSize     int
@@ -52,28 +53,16 @@ func (acct *AccountView) UiConfig() *config.UIConfig {
 	if dirlist := acct.Directories(); dirlist != nil {
 		return dirlist.UiConfig("")
 	}
-	return acct.uiConf
+	return config.Ui.ForAccount(acct.acct.Name)
 }
 
 func NewAccountView(
 	acct *config.AccountConfig, deferLoop chan struct{},
 ) (*AccountView, error) {
-	acctUiConf := config.Ui.ForAccount(acct.Name)
-
 	view := &AccountView{
-		acct:   acct,
-		uiConf: acctUiConf,
+		acct: acct,
 	}
 
-	view.grid = ui.NewGrid().Rows([]ui.GridSpec{
-		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
-	}).Columns([]ui.GridSpec{
-		{Strategy: ui.SIZE_EXACT, Size: func() int {
-			return view.UiConfig().SidebarWidth
-		}},
-		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
-	})
-
 	worker, err := worker.NewWorker(acct.Source, acct.Name)
 	if err != nil {
 		SetError(fmt.Sprintf("%s: %s", acct.Name, err))
@@ -83,17 +72,10 @@ func NewAccountView(
 	view.worker = worker
 
 	view.dirlist = NewDirectoryList(acct, worker)
-	if acctUiConf.SidebarWidth > 0 {
-		view.grid.AddChild(ui.NewBordered(view.dirlist, ui.BORDER_RIGHT, acctUiConf))
-	}
 
 	view.msglist = NewMessageList(view)
-	view.grid.AddChild(view.msglist).At(0, 1)
 
-	view.dirlist.OnVirtualNode(func() {
-		view.msglist.SetStore(nil)
-		view.Invalidate()
-	})
+	view.Configure()
 
 	go func() {
 		defer log.PanicHandler()
@@ -115,6 +97,40 @@ func NewAccountView(
 	return view, nil
 }
 
+func (acct *AccountView) Configure() {
+	acct.dirlist.OnVirtualNode(func() {
+		acct.msglist.SetStore(nil)
+		acct.Invalidate()
+	})
+	sidebar := acct.UiConfig().SidebarWidth
+	acct.grid = ui.NewGrid().Rows([]ui.GridSpec{
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
+	}).Columns([]ui.GridSpec{
+		{Strategy: ui.SIZE_EXACT, Size: func() int {
+			return sidebar
+		}},
+		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
+	})
+	if sidebar > 0 {
+		acct.grid.AddChild(ui.NewBordered(acct.dirlist, ui.BORDER_RIGHT, acct.UiConfig()))
+	}
+	acct.grid.AddChild(acct.msglist).At(0, 1)
+	acct.setTitle()
+
+	// handle splits
+	if acct.split != nil {
+		acct.split.Close()
+	}
+	splitDirection := acct.splitDir
+	acct.splitDir = config.SPLIT_NONE
+	switch splitDirection {
+	case config.SPLIT_HORIZONTAL:
+		acct.Split(acct.SplitSize())
+	case config.SPLIT_VERTICAL:
+		acct.Vsplit(acct.SplitSize())
+	}
+}
+
 func (acct *AccountView) SetStatus(setters ...state.SetStateFunc) {
 	for _, fn := range setters {
 		fn(&acct.state, acct.SelectedDirectory())
@@ -180,6 +196,13 @@ func (acct *AccountView) Directories() DirectoryLister {
 	return acct.dirlist
 }
 
+func (acct *AccountView) SetDirectories(d DirectoryLister) {
+	if acct.grid != nil {
+		acct.grid.ReplaceChild(acct.dirlist, d)
+	}
+	acct.dirlist = d
+}
+
 func (acct *AccountView) Labels() []string {
 	return acct.labels
 }
@@ -217,7 +240,7 @@ func (acct *AccountView) SelectedMessage() (*models.MessageInfo, error) {
 	return msg, nil
 }
 
-func (acct *AccountView) MarkedMessages() ([]uint32, error) {
+func (acct *AccountView) MarkedMessages() ([]models.UID, error) {
 	if store := acct.Store(); store != nil {
 		return store.Marker().Marked(), nil
 	}
@@ -248,16 +271,12 @@ func (acct *AccountView) newStore(name string) *lib.MessageStore {
 		role = string(dir.Role)
 	}
 	backend := acct.AccountConfig().Backend
-	store := lib.NewMessageStore(acct.worker,
-		acct.sortCriteria(uiConf),
-		uiConf.ThreadingEnabled,
-		uiConf.ForceClientThreads,
-		uiConf.ClientThreadsDelay,
-		uiConf.SelectLast,
-		uiConf.ThreadingBySubject,
-		uiConf.ReverseOrder,
-		uiConf.ReverseThreadOrder,
-		uiConf.SortThreadSiblings,
+	store := lib.NewMessageStore(acct.worker, name,
+		func() *config.UIConfig {
+			return config.Ui.
+				ForAccount(acct.Name()).
+				ForFolder(name)
+		},
 		func(msg *models.MessageInfo) {
 			err := hooks.RunHook(&hooks.MailReceived{
 				Account: acct.Name(),
@@ -320,9 +339,29 @@ func (acct *AccountView) newStore(name string) *lib.MessageStore {
 				PushError(msg)
 			}
 		},
-		acct.updateSplitView,
-		acct.dirlist.UiConfig(name).ThreadContext,
+		func(msg *models.MessageInfo) {
+			acct.updateSplitView(msg)
+
+			auto := false
+			if c := acct.AccountConfig(); c != nil {
+				r, ok := c.Params["pama-auto-switch"]
+				if ok {
+					if strings.ToLower(r) == "true" {
+						auto = true
+					}
+				}
+			}
+			if !auto {
+				return
+			}
+			var name string
+			if msg != nil && msg.Envelope != nil {
+				name = pama.FromSubject(msg.Envelope.Subject)
+			}
+			pama.DebouncedSwitchProject(name)
+		},
 	)
+	store.Configure(acct.SortCriteria(uiConf))
 	store.SetMarker(marker.New(store))
 	return store
 }
@@ -456,7 +495,7 @@ func (acct *AccountView) onMessage(msg types.WorkerMessage) {
 	acct.setTitle()
 }
 
-func (acct *AccountView) updateDirCounts(destination string, uids []uint32) {
+func (acct *AccountView) updateDirCounts(destination string, uids []models.UID) {
 	// Only update the destination destDir if it is initialized
 	if destDir := acct.dirlist.Directory(destination); destDir != nil {
 		var recent, unseen int
@@ -490,7 +529,7 @@ func (acct *AccountView) updateDirCounts(destination string, uids []uint32) {
 	}
 }
 
-func (acct *AccountView) sortCriteria(uiConf *config.UIConfig) []*types.SortCriterion {
+func (acct *AccountView) SortCriteria(uiConf *config.UIConfig) []*types.SortCriterion {
 	if uiConf == nil {
 		return nil
 	}
@@ -506,7 +545,7 @@ func (acct *AccountView) sortCriteria(uiConf *config.UIConfig) []*types.SortCrit
 }
 
 func (acct *AccountView) GetSortCriteria() []*types.SortCriterion {
-	return acct.sortCriteria(acct.UiConfig())
+	return acct.SortCriteria(acct.UiConfig())
 }
 
 func (acct *AccountView) CheckMail() {
@@ -593,18 +632,19 @@ func (acct *AccountView) closeSplit() {
 		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	})
 
-	acct.grid.AddChild(ui.NewBordered(acct.dirlist, ui.BORDER_RIGHT, acct.uiConf))
+	acct.grid.AddChild(ui.NewBordered(acct.dirlist, ui.BORDER_RIGHT, acct.UiConfig()))
 	acct.grid.AddChild(acct.msglist).At(0, 1)
 	ui.Invalidate()
 }
 
 func (acct *AccountView) updateSplitView(msg *models.MessageInfo) {
+	uiConf := acct.UiConfig()
 	if !acct.splitLoaded {
-		switch acct.uiConf.MessageListSplit.Direction {
+		switch uiConf.MessageListSplit.Direction {
 		case config.SPLIT_HORIZONTAL:
-			acct.Split(acct.uiConf.MessageListSplit.Size)
+			acct.Split(uiConf.MessageListSplit.Size)
 		case config.SPLIT_VERTICAL:
-			acct.Vsplit(acct.uiConf.MessageListSplit.Size)
+			acct.Vsplit(uiConf.MessageListSplit.Size)
 		}
 		acct.splitLoaded = true
 	}
@@ -625,7 +665,12 @@ func (acct *AccountView) updateSplitView(msg *models.MessageInfo) {
 					PushError(err.Error())
 					return
 				}
-				acct.split = NewMessageViewer(acct, view)
+				viewer, err := NewMessageViewer(acct, view)
+				if err != nil {
+					PushError(err.Error())
+					return
+				}
+				acct.split = viewer
 				switch acct.splitDir {
 				case config.SPLIT_HORIZONTAL:
 					acct.grid.AddChild(acct.split).At(1, 1)
@@ -669,11 +714,15 @@ func (acct *AccountView) Split(n int) {
 		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	})
 
-	acct.grid.AddChild(ui.NewBordered(acct.dirlist, ui.BORDER_RIGHT, acct.uiConf)).Span(2, 1)
-	acct.grid.AddChild(ui.NewBordered(acct.msglist, ui.BORDER_BOTTOM, acct.uiConf)).At(0, 1)
-	acct.split = NewMessageViewer(acct, nil)
+	acct.grid.AddChild(ui.NewBordered(acct.dirlist, ui.BORDER_RIGHT, acct.UiConfig())).Span(2, 1)
+	acct.grid.AddChild(ui.NewBordered(acct.msglist, ui.BORDER_BOTTOM, acct.UiConfig())).At(0, 1)
+	acct.split, _ = NewMessageViewer(acct, nil)
 	acct.grid.AddChild(acct.split).At(1, 1)
-	acct.updateSplitView(acct.msglist.Selected())
+	msg, err := acct.SelectedMessage()
+	if err != nil {
+		log.Debugf("split: load message error: %v", err)
+	}
+	acct.updateSplitView(msg)
 }
 
 // Vsplit splits the message list view vertically. The message list will be n
@@ -694,11 +743,15 @@ func (acct *AccountView) Vsplit(n int) {
 		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	})
 
-	acct.grid.AddChild(ui.NewBordered(acct.dirlist, ui.BORDER_RIGHT, acct.uiConf)).At(0, 0)
-	acct.grid.AddChild(ui.NewBordered(acct.msglist, ui.BORDER_RIGHT, acct.uiConf)).At(0, 1)
-	acct.split = NewMessageViewer(acct, nil)
+	acct.grid.AddChild(ui.NewBordered(acct.dirlist, ui.BORDER_RIGHT, acct.UiConfig())).At(0, 0)
+	acct.grid.AddChild(ui.NewBordered(acct.msglist, ui.BORDER_RIGHT, acct.UiConfig())).At(0, 1)
+	acct.split, _ = NewMessageViewer(acct, nil)
 	acct.grid.AddChild(acct.split).At(0, 2)
-	acct.updateSplitView(acct.msglist.Selected())
+	msg, err := acct.SelectedMessage()
+	if err != nil {
+		log.Debugf("split: load message error: %v", err)
+	}
+	acct.updateSplitView(msg)
 }
 
 // setTitle executes the title template and sets the tab title
@@ -711,9 +764,10 @@ func (acct *AccountView) setTitle() {
 	data.SetAccount(acct.acct)
 	data.SetFolder(acct.Directories().SelectedDirectory())
 	data.SetRUE(acct.dirlist.List(), acct.dirlist.GetRUECount)
+	data.SetState(&acct.state)
 
 	var buf bytes.Buffer
-	err := templates.Render(acct.uiConf.TabTitleAccount, &buf, data.Data())
+	err := templates.Render(acct.UiConfig().TabTitleAccount, &buf, data.Data())
 	if err != nil {
 		acct.PushError(err)
 		return
