@@ -1,6 +1,7 @@
 package app
 
 import (
+	"context"
 	"errors"
 	"fmt"
 	"io"
@@ -11,7 +12,7 @@ import (
 	"time"
 	"unicode"
 
-	"git.sr.ht/~rjarry/go-opt"
+	"git.sr.ht/~rjarry/go-opt/v2"
 	"git.sr.ht/~rockorager/vaxis"
 	"github.com/ProtonMail/go-crypto/openpgp"
 	"github.com/emersion/go-message/mail"
@@ -29,7 +30,7 @@ type Aerc struct {
 	accounts    map[string]*AccountView
 	cmd         func(string, *config.AccountConfig, *models.MessageInfo) error
 	cmdHistory  lib.History
-	complete    func(cmd string) ([]string, string)
+	complete    func(ctx context.Context, cmd string) ([]opt.Completion, string)
 	focused     ui.Interactive
 	grid        *ui.Grid
 	simulating  int
@@ -54,10 +55,16 @@ type Choice struct {
 func (aerc *Aerc) Init(
 	crypto crypto.Provider,
 	cmd func(string, *config.AccountConfig, *models.MessageInfo) error,
-	complete func(cmd string) ([]string, string), cmdHistory lib.History,
+	complete func(ctx context.Context, cmd string) ([]opt.Completion, string), cmdHistory lib.History,
 	deferLoop chan struct{},
 ) {
-	tabs := ui.NewTabs(config.Ui)
+	tabs := ui.NewTabs(func(d ui.Drawable) *config.UIConfig {
+		acct := aerc.account(d)
+		if acct != nil {
+			return config.Ui.ForAccount(acct.Name())
+		}
+		return config.Ui
+	})
 
 	statusbar := ui.NewStack(config.Ui)
 	statusline := &StatusLine{}
@@ -88,17 +95,17 @@ func (aerc *Aerc) Init(
 	for _, acct := range config.Accounts {
 		view, err := NewAccountView(acct, deferLoop)
 		if err != nil {
-			tabs.Add(errorScreen(err.Error()), acct.Name, nil)
+			tabs.Add(errorScreen(err.Error()), acct.Name, false)
 		} else {
 			aerc.accounts[acct.Name] = view
-			view.tab = tabs.Add(view, acct.Name, view.UiConfig())
+			view.tab = tabs.Add(view, acct.Name, false)
 		}
 	}
 
 	if len(config.Accounts) == 0 {
 		wizard := NewAccountWizard()
 		wizard.Focus(true)
-		aerc.NewTab(wizard, "New account")
+		aerc.NewTab(wizard, "New account", false)
 	}
 
 	tabs.Select(0)
@@ -184,16 +191,15 @@ func (aerc *Aerc) Draw(ctx *ui.Context) {
 	}
 	aerc.grid.Draw(ctx)
 	if aerc.dialog != nil {
-		if w, h := ctx.Width(), ctx.Height(); w > 8 && h > 4 {
-			if d, ok := aerc.dialog.(Dialog); ok {
-				xstart, width := d.ContextWidth()
-				ystart, height := d.ContextHeight()
-				aerc.dialog.Draw(
-					ctx.Subcontext(xstart(w), ystart(h),
-						width(w), height(h)))
-			} else {
-				aerc.dialog.Draw(ctx.Subcontext(4, h/2-2, w-8, 4))
-			}
+		w, h := ctx.Width(), ctx.Height()
+		if d, ok := aerc.dialog.(Dialog); ok {
+			xstart, width := d.ContextWidth()
+			ystart, height := d.ContextHeight()
+			aerc.dialog.Draw(
+				ctx.Subcontext(xstart(w), ystart(h),
+					width(w), height(h)))
+		} else if w > 8 && h > 4 {
+			aerc.dialog.Draw(ctx.Subcontext(4, h/2-2, w-8, 4))
 		}
 	}
 }
@@ -312,8 +318,8 @@ func (aerc *Aerc) simulate(strokes []config.KeyStroke) {
 	aerc.simulating -= 1
 	if exline, ok := aerc.focused.(*ExLine); ok {
 		// we are still focused on the exline, turn on tab complete
-		exline.TabComplete(func(cmd string) ([]string, string) {
-			return aerc.complete(cmd)
+		exline.TabComplete(func(ctx context.Context, cmd string) ([]opt.Completion, string) {
+			return aerc.complete(ctx, cmd)
 		})
 		if complete {
 			// force completion now
@@ -323,6 +329,13 @@ func (aerc *Aerc) simulate(strokes []config.KeyStroke) {
 }
 
 func (aerc *Aerc) Event(event vaxis.Event) bool {
+	if config.General.QuakeMode {
+		if e, ok := event.(vaxis.Key); ok && e.MatchString("F1") {
+			ToggleQuake()
+			return true
+		}
+	}
+
 	if aerc.dialog != nil {
 		return aerc.dialog.Event(event)
 	}
@@ -503,12 +516,8 @@ func (aerc *Aerc) SelectedTab() *ui.Tab {
 	return aerc.tabs.Selected()
 }
 
-func (aerc *Aerc) NewTab(clickable ui.Drawable, name string) *ui.Tab {
-	uiConf := config.Ui
-	if acct := aerc.account(clickable); acct != nil {
-		uiConf = acct.UiConfig()
-	}
-	tab := aerc.tabs.Add(clickable, name, uiConf)
+func (aerc *Aerc) NewTab(clickable ui.Drawable, name string, background bool) *ui.Tab {
+	tab := aerc.tabs.Add(clickable, name, background)
 	aerc.UpdateStatus()
 	return tab
 }
@@ -625,14 +634,12 @@ func (aerc *Aerc) focus(item ui.Interactive) {
 
 func (aerc *Aerc) BeginExCommand(cmd string) {
 	previous := aerc.focused
-	var tabComplete func(string) ([]string, string)
+	var tabComplete func(context.Context, string) ([]opt.Completion, string)
 	if aerc.simulating != 0 {
 		// Don't try to draw completions for simulated events
 		tabComplete = nil
 	} else {
-		tabComplete = func(cmd string) ([]string, string) {
-			return aerc.complete(cmd)
-		}
+		tabComplete = aerc.complete
 	}
 	exline := NewExLine(cmd, func(cmd string) {
 		err := aerc.cmd(cmd, nil, nil)
@@ -665,7 +672,7 @@ func (aerc *Aerc) RegisterPrompt(prompt string, cmd string) {
 		if err != nil {
 			aerc.PushError(err.Error())
 		}
-	}, func(cmd string) ([]string, string) {
+	}, func(ctx context.Context, cmd string) ([]opt.Completion, string) {
 		return nil, "" // TODO: completions
 	})
 	aerc.prompts.Push(p)
@@ -692,7 +699,7 @@ func (aerc *Aerc) RegisterChoices(choices []Choice) {
 		if err != nil {
 			aerc.PushError(err.Error())
 		}
-	}, func(cmd string) ([]string, string) {
+	}, func(ctx context.Context, cmd string) ([]opt.Completion, string) {
 		return nil, "" // TODO: completions
 	})
 	aerc.prompts.Push(p)
@@ -806,7 +813,7 @@ func (aerc *Aerc) mailto(addr *url.URL) error {
 	if to == nil {
 		composer.FocusEditor("to")
 	}
-	composer.Tab = aerc.NewTab(composer, title)
+	composer.Tab = aerc.NewTab(composer, title, false)
 
 	for _, file := range attachments {
 		composer.AddAttachment(file)
@@ -829,16 +836,16 @@ func (aerc *Aerc) mbox(source string) error {
 	acctConf.Default = "INBOX"
 	acctConf.Archive = "Archive"
 	acctConf.Postpone = "Drafts"
-	acctConf.CopyTo = "Sent"
+	acctConf.CopyTo = []string{"Sent"}
 
 	defer ui.Invalidate()
 
 	mboxView, err := NewAccountView(&acctConf, nil)
 	if err != nil {
-		aerc.NewTab(errorScreen(err.Error()), acctConf.Name)
+		aerc.NewTab(errorScreen(err.Error()), acctConf.Name, false)
 	} else {
 		aerc.accounts[acctConf.Name] = mboxView
-		aerc.NewTab(mboxView, acctConf.Name)
+		aerc.NewTab(mboxView, acctConf.Name, false)
 	}
 	return nil
 }
@@ -931,7 +938,7 @@ func (aerc *Aerc) isExKey(key vaxis.Key, exKey config.KeyStroke) bool {
 	return key.Matches(exKey.Key, exKey.Modifiers)
 }
 
-// CmdFallbackSearch checks cmds for the first executable availabe in PATH. An error is
+// CmdFallbackSearch checks cmds for the first executable available in PATH. An error is
 // returned if none are found
 func CmdFallbackSearch(cmds []string, silent bool) (string, error) {
 	var tried []string
