@@ -1,6 +1,7 @@
 package app
 
 import (
+	"context"
 	"time"
 
 	"git.sr.ht/~rjarry/aerc/config"
@@ -10,6 +11,7 @@ import (
 	"git.sr.ht/~rjarry/aerc/lib/ui"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
+	"git.sr.ht/~rjarry/go-opt/v2"
 	"github.com/ProtonMail/go-crypto/openpgp"
 )
 
@@ -18,7 +20,7 @@ var aerc Aerc
 func Init(
 	crypto crypto.Provider,
 	cmd func(string, *config.AccountConfig, *models.MessageInfo) error,
-	complete func(cmd string) ([]string, string), history lib.History,
+	complete func(ctx context.Context, cmd string) ([]opt.Completion, string), history lib.History,
 	deferLoop chan struct{},
 ) {
 	aerc.Init(crypto, cmd, complete, history, deferLoop)
@@ -45,20 +47,28 @@ func PrevAccount() (*AccountView, error)        { return aerc.PrevAccount() }
 func SelectedAccount() *AccountView             { return aerc.SelectedAccount() }
 func SelectedAccountUiConfig() *config.UIConfig { return aerc.SelectedAccountUiConfig() }
 
-func NextTab()                                          { aerc.NextTab() }
-func PrevTab()                                          { aerc.PrevTab() }
-func PinTab()                                           { aerc.PinTab() }
-func UnpinTab()                                         { aerc.UnpinTab() }
-func MoveTab(i int, relative bool)                      { aerc.MoveTab(i, relative) }
-func TabNames() []string                                { return aerc.TabNames() }
-func SelectTab(name string) bool                        { return aerc.SelectTab(name) }
-func SelectPreviousTab() bool                           { return aerc.SelectPreviousTab() }
-func SelectedTab() *ui.Tab                              { return aerc.SelectedTab() }
-func SelectedTabContent() ui.Drawable                   { return aerc.SelectedTabContent() }
-func SelectTabIndex(index int) bool                     { return aerc.SelectTabIndex(index) }
-func SelectTabAtOffset(offset int)                      { aerc.SelectTabAtOffset(offset) }
-func RemoveTab(tab ui.Drawable, closeContent bool)      { aerc.RemoveTab(tab, closeContent) }
-func NewTab(clickable ui.Drawable, name string) *ui.Tab { return aerc.NewTab(clickable, name) }
+func NextTab()                                     { aerc.NextTab() }
+func PrevTab()                                     { aerc.PrevTab() }
+func PinTab()                                      { aerc.PinTab() }
+func UnpinTab()                                    { aerc.UnpinTab() }
+func MoveTab(i int, relative bool)                 { aerc.MoveTab(i, relative) }
+func TabNames() []string                           { return aerc.TabNames() }
+func GetTab(i int) *ui.Tab                         { return aerc.tabs.Get(i) }
+func SelectTab(name string) bool                   { return aerc.SelectTab(name) }
+func SelectPreviousTab() bool                      { return aerc.SelectPreviousTab() }
+func SelectedTab() *ui.Tab                         { return aerc.SelectedTab() }
+func SelectedTabContent() ui.Drawable              { return aerc.SelectedTabContent() }
+func SelectTabIndex(index int) bool                { return aerc.SelectTabIndex(index) }
+func SelectTabAtOffset(offset int)                 { aerc.SelectTabAtOffset(offset) }
+func RemoveTab(tab ui.Drawable, closeContent bool) { aerc.RemoveTab(tab, closeContent) }
+func NewTab(clickable ui.Drawable, name string) *ui.Tab {
+	return aerc.NewTab(clickable, name, false)
+}
+
+func NewBackgroundTab(clickable ui.Drawable, name string) *ui.Tab {
+	return aerc.NewTab(clickable, name, true)
+}
+
 func ReplaceTab(tabSrc ui.Drawable, tabTarget ui.Drawable, name string, closeSrc bool) {
 	aerc.ReplaceTab(tabSrc, tabTarget, name, closeSrc)
 }
