@@ -9,7 +9,6 @@ import (
 	"os"
 	"os/exec"
 	"path/filepath"
-	"sort"
 	"strconv"
 	"strings"
 	"sync"
@@ -888,6 +887,30 @@ func (c *Composer) parseEmbeddedHeader() (*mail.Header, error) {
 	} else if err != nil {
 		return nil, fmt.Errorf("mail.ReadMessage: %w", err)
 	}
+
+	// merge repeated to, cc, and bcc headers into a single one of each
+	for _, key := range []string{"To", "Cc", "Bcc"} {
+		fields := msg.Header.FieldsByKey(key)
+		if fields.Len() <= 1 {
+			continue
+		}
+		var addrs []*mail.Address
+		for fields.Next() {
+			if strings.TrimSpace(fields.Value()) == "" {
+				continue
+			}
+			al, err := mail.ParseAddressList(fields.Value())
+			if err != nil {
+				return nil, fmt.Errorf(
+					"%s: cannot parse address list: %w", key, err)
+			}
+			addrs = append(addrs, al...)
+		}
+		msg.Header.SetAddressList(key, addrs)
+		PushWarning(fmt.Sprintf(
+			"Multiple %s headers found; merged in a single one.", key))
+	}
+
 	return &msg.Header, nil
 }
 
@@ -1651,64 +1674,58 @@ type reviewMessage struct {
 	grid     *ui.Grid
 }
 
+var defaultAnnotations = map[string]string{
+	":send<enter>":     "Send",
+	":edit<enter>":     "Edit (body and headers)",
+	":attach<space>":   "Add attachment",
+	":detach<space>":   "Remove attachment",
+	":postpone<enter>": "Postpone",
+	":preview<enter>":  "Preview message",
+	":abort<enter>":    "Abort (discard message, no confirmation)",
+	":choose -o d discard abort -o p postpone postpone<enter>": "Abort or postpone",
+}
+
 func newReviewMessage(composer *Composer, err error) *reviewMessage {
 	bindings := config.Binds.ComposeReview.ForAccount(
 		composer.acctConfig.Name,
 	)
 	bindings = bindings.ForFolder(composer.SelectedDirectory())
 
-	const maxInputWidth = 6
 	type reviewCmd struct {
+		input      string
 		output     string
 		annotation string
-		input      string
 	}
 
-	reviewCmds := []reviewCmd{
-		{":send<enter>", "Send", ""},
-		{":edit<enter>", "Edit (body and headers)", ""},
-		{":attach<space>", "Add attachment", ""},
-		{":detach<space>", "Remove attachment", ""},
-		{":postpone<enter>", "Postpone", ""},
-		{":preview<enter>", "Preview message", ""},
-		{":abort<enter>", "Abort (discard message, no confirmation)", ""},
-		{":choose -o d discard abort -o p postpone postpone<enter>", "Abort or postpone", ""},
-	}
-	knownCommands := len(reviewCmds)
-	var actions []string
+	var reviewCmds []reviewCmd
+
 	for _, binding := range bindings.Bindings {
+		if binding.Annotation == "-" {
+			// explicitly hidden by user
+			continue
+		}
+
 		inputs := config.FormatKeyStrokes(binding.Input)
 		outputs := config.FormatKeyStrokes(binding.Output)
-		found := false
-		for i, rcmd := range reviewCmds {
-			if outputs == rcmd.output {
-				found = true
-				if reviewCmds[i].input == "" {
-					reviewCmds[i].input = inputs
-				} else {
-					reviewCmds[i].input += ", " + inputs
+		annotation := binding.Annotation
+		if annotation == "" {
+			for i := range reviewCmds {
+				r := &reviewCmds[i]
+				if r.output == outputs {
+					// aliased action with a different binding
+					r.input += ", " + inputs
+					goto next
 				}
-				if binding.Annotation != "" {
-					// overwrite default description with
-					// user annotations if present
-					reviewCmds[i].annotation = binding.Annotation
-				}
-				break
 			}
+			annotation = defaultAnnotations[outputs]
 		}
-		if !found {
-			rcmd := reviewCmd{
-				output:     outputs,
-				annotation: binding.Annotation,
-				input:      inputs,
-			}
-			reviewCmds = append(reviewCmds, rcmd)
-		}
+		reviewCmds = append(reviewCmds, reviewCmd{
+			input:      inputs,
+			output:     outputs,
+			annotation: annotation,
+		})
+	next:
 	}
-	unknownCommands := reviewCmds[knownCommands:]
-	sort.Slice(unknownCommands, func(i, j int) bool {
-		return unknownCommands[i].input < unknownCommands[j].input
-	})
 
 	longest := 0
 	for _, rcmd := range reviewCmds {
@@ -1717,17 +1734,17 @@ func newReviewMessage(composer *Composer, err error) *reviewMessage {
 		}
 	}
 
+	const maxInputWidth = 6
 	width := longest
 	if longest < maxInputWidth {
 		width = maxInputWidth
 	}
 	widthstr := strconv.Itoa(width)
 
+	var actions []string
 	for _, rcmd := range reviewCmds {
-		if rcmd.input != "" {
-			actions = append(actions, fmt.Sprintf("  %-"+widthstr+"s  %-40s  %s",
-				rcmd.input, rcmd.annotation, rcmd.output))
-		}
+		actions = append(actions, fmt.Sprintf("  %-"+widthstr+"s  %-40s  %s",
+			rcmd.input, rcmd.annotation, rcmd.output))
 	}
 
 	spec := []ui.GridSpec{
