@@ -4,6 +4,7 @@ import (
 	"bytes"
 	"errors"
 	"fmt"
+	"image"
 	"io"
 	"os"
 	"os/exec"
@@ -12,25 +13,43 @@ import (
 
 	"github.com/danwakefield/fnmatch"
 	"github.com/emersion/go-message/textproto"
-	"github.com/gdamore/tcell/v2"
 	"github.com/mattn/go-runewidth"
 
 	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/auth"
 	"git.sr.ht/~rjarry/aerc/lib/format"
+	"git.sr.ht/~rjarry/aerc/lib/log"
 	"git.sr.ht/~rjarry/aerc/lib/parse"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
-	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
-	"git.sr.ht/~rjarry/go-opt"
+	"git.sr.ht/~rjarry/go-opt/v2"
+	"git.sr.ht/~rockorager/vaxis"
+	"git.sr.ht/~rockorager/vaxis/widgets/align"
+
+	// Image support
+	_ "image/jpeg"
+	_ "image/png"
+
+	_ "golang.org/x/image/bmp"
+	_ "golang.org/x/image/tiff"
+	_ "golang.org/x/image/webp"
 )
 
+// All imported image types need to be explicitly stated here. We want to check
+// if we _can_ display something before we download it
+var supportedImageTypes = []string{
+	"image/jpeg",
+	"image/png",
+	"image/bmp",
+	"image/tiff",
+	"image/webp",
+}
+
 var _ ProvidesMessages = (*MessageViewer)(nil)
 
 type MessageViewer struct {
 	acct     *AccountView
-	err      error
 	grid     *ui.Grid
 	switcher *PartSwitcher
 	msg      lib.MessageView
@@ -39,12 +58,9 @@ type MessageViewer struct {
 
 func NewMessageViewer(
 	acct *AccountView, msg lib.MessageView,
-) *MessageViewer {
+) (*MessageViewer, error) {
 	if msg == nil {
-		return &MessageViewer{
-			acct: acct,
-			err:  fmt.Errorf("(no message selected)"),
-		}
+		return &MessageViewer{acct: acct}, nil
 	}
 	hf := HeaderLayoutFilter{
 		layout: HeaderLayout(config.Viewer.HeaderLayout),
@@ -110,13 +126,7 @@ func NewMessageViewer(
 	switcher := &PartSwitcher{}
 	err := createSwitcher(acct, switcher, msg)
 	if err != nil {
-		return &MessageViewer{
-			acct:     acct,
-			err:      err,
-			grid:     grid,
-			msg:      msg,
-			uiConfig: acct.UiConfig(),
-		}
+		return nil, err
 	}
 
 	borderStyle := acct.UiConfig().GetStyle(config.STYLE_BORDER)
@@ -141,7 +151,7 @@ func NewMessageViewer(
 	}
 	switcher.uiConfig = mv.uiConfig
 
-	return mv
+	return mv, nil
 }
 
 func fmtHeader(msg *models.MessageInfo, header string,
@@ -158,6 +168,8 @@ func fmtHeader(msg *models.MessageInfo, header string,
 	switch header {
 	case "From":
 		return format.FormatAddresses(msg.Envelope.From)
+	case "Sender":
+		return format.FormatAddresses(msg.Envelope.Sender)
 	case "To":
 		return format.FormatAddresses(msg.Envelope.To)
 	case "Cc":
@@ -256,17 +268,17 @@ func createSwitcher(
 }
 
 func (mv *MessageViewer) Draw(ctx *ui.Context) {
-	if mv.err != nil {
+	if mv.switcher == nil {
 		style := mv.acct.UiConfig().GetStyle(config.STYLE_DEFAULT)
 		ctx.Fill(0, 0, ctx.Width(), ctx.Height(), ' ', style)
-		ctx.Printf(0, 0, style, "%s", mv.err.Error())
+		ctx.Printf(0, 0, style, "%s", "(no message selected)")
 		return
 	}
 	mv.grid.Draw(ctx)
 }
 
-func (mv *MessageViewer) MouseEvent(localX int, localY int, event tcell.Event) {
-	if mv.err != nil {
+func (mv *MessageViewer) MouseEvent(localX int, localY int, event vaxis.Event) {
+	if mv.switcher == nil {
 		return
 	}
 	mv.grid.MouseEvent(localX, localY, event)
@@ -313,11 +325,14 @@ func (mv *MessageViewer) SelectedMessage() (*models.MessageInfo, error) {
 	return mv.msg.MessageInfo(), nil
 }
 
-func (mv *MessageViewer) MarkedMessages() ([]uint32, error) {
+func (mv *MessageViewer) MarkedMessages() ([]models.UID, error) {
 	return mv.acct.MarkedMessages()
 }
 
 func (mv *MessageViewer) ToggleHeaders() {
+	if mv.switcher == nil {
+		return
+	}
 	switcher := mv.switcher
 	switcher.Cleanup()
 	config.Viewer.ShowHeaders = !config.Viewer.ShowHeaders
@@ -334,6 +349,9 @@ func (mv *MessageViewer) ToggleKeyPassthrough() bool {
 }
 
 func (mv *MessageViewer) SelectedMessagePart() *PartInfo {
+	if mv.switcher == nil {
+		return nil
+	}
 	part := mv.switcher.SelectedPart()
 	return &PartInfo{
 		Index: part.index,
@@ -344,6 +362,9 @@ func (mv *MessageViewer) SelectedMessagePart() *PartInfo {
 }
 
 func (mv *MessageViewer) AttachmentParts(all bool) []*PartInfo {
+	if mv.switcher == nil {
+		return nil
+	}
 	return mv.switcher.AttachmentParts(all)
 }
 
@@ -377,16 +398,23 @@ func (mv *MessageViewer) Close() {
 	}
 }
 
-func (mv *MessageViewer) Event(event tcell.Event) bool {
-	return mv.switcher.Event(event)
+func (mv *MessageViewer) Event(event vaxis.Event) bool {
+	if mv.switcher != nil {
+		return mv.switcher.Event(event)
+	}
+	return false
 }
 
 func (mv *MessageViewer) Focus(focus bool) {
-	mv.switcher.Focus(focus)
+	if mv.switcher != nil {
+		mv.switcher.Focus(focus)
+	}
 }
 
 func (mv *MessageViewer) Show(visible bool) {
-	mv.switcher.Show(visible)
+	if mv.switcher != nil {
+		mv.switcher.Show(visible)
+	}
 }
 
 type PartViewer struct {
@@ -405,6 +433,11 @@ type PartViewer struct {
 	noFilter   *ui.Grid
 	uiConfig   *config.UIConfig
 	copying    int32
+	inlineImg  bool
+	image      image.Image
+	graphic    vaxis.Image
+	width      int
+	height     int
 
 	links []string
 }
@@ -421,14 +454,6 @@ func NewPartViewer(
 		pagerin io.WriteCloser
 		term    *Terminal
 	)
-	pagerCmd, err := CmdFallbackSearch(config.PagerCmds(), false)
-	if err != nil {
-		acct.PushError(fmt.Errorf("could not start pager: %w", err))
-		return nil, err
-	}
-	cmd := opt.SplitArgs(pagerCmd)
-	pager = exec.Command(cmd[0], cmd[1:]...)
-
 	info := msg.MessageInfo()
 	mime := part.FullMIMEType()
 
@@ -455,10 +480,27 @@ func NewPartViewer(
 			if f.Regex.Match([]byte(header)) {
 				filter = exec.Command("sh", "-c", f.Command)
 			}
+		case config.FILTER_FILENAME:
+			if f.Regex.Match([]byte(part.DispositionParams["filename"])) {
+				filter = exec.Command("sh", "-c", f.Command)
+				log.Tracef("command %v", f.Command)
+			}
 		}
-		if filter != nil {
+		if filter == nil {
+			continue
+		}
+		if !f.NeedsPager {
+			pager = filter
 			break
 		}
+		pagerCmd, err := CmdFallbackSearch(config.PagerCmds(), false)
+		if err != nil {
+			acct.PushError(fmt.Errorf("could not start pager: %w", err))
+			return nil, err
+		}
+		cmd := opt.SplitArgs(pagerCmd)
+		pager = exec.Command(cmd[0], cmd[1:]...)
+		break
 	}
 	var noFilter *ui.Grid
 	if filter != nil {
@@ -488,8 +530,14 @@ func NewPartViewer(
 		if config.General.EnableOSC8 {
 			filter.Env = append(filter.Env, "AERC_OSC8_URLS=1")
 		}
-		log.Debugf("<%s> part=%v %s: %v | %v",
-			info.Envelope.MessageId, curindex, mime, filter, pager)
+		if pager == filter {
+			log.Debugf("<%s> part=%v %s: %v",
+				info.Envelope.MessageId, curindex, mime, filter)
+		} else {
+			log.Debugf("<%s> part=%v %s: %v | %v",
+				info.Envelope.MessageId, curindex, mime, filter, pager)
+		}
+		var err error
 		if pagerin, err = pager.StdinPipe(); err != nil {
 			return nil, err
 		}
@@ -524,120 +572,150 @@ func NewPartViewer(
 		uiConfig:   acct.UiConfig(),
 	}
 
-	if term != nil {
-		term.OnStart = func() {
-			pv.attemptCopy()
-		}
-	}
-
 	return pv, nil
 }
 
 func (pv *PartViewer) SetSource(reader io.Reader) {
 	pv.source = reader
-	pv.attemptCopy()
+	switch pv.inlineImg {
+	case true:
+		pv.decodeImage()
+	default:
+		pv.attemptCopy()
+	}
+}
+
+func (pv *PartViewer) decodeImage() {
+	atomic.StoreInt32(&pv.copying, copying)
+	go func() {
+		defer log.PanicHandler()
+		defer pv.Invalidate()
+		defer atomic.StoreInt32(&pv.copying, 0)
+		img, _, err := image.Decode(pv.source)
+		if err != nil {
+			log.Errorf("error decoding image: %v", err)
+			return
+		}
+		pv.image = img
+	}()
 }
 
 func (pv *PartViewer) attemptCopy() {
 	if pv.source == nil ||
 		pv.filter == nil ||
-		atomic.LoadInt32(&pv.copying) == copying {
+		atomic.SwapInt32(&pv.copying, copying) == copying {
 		return
 	}
-	atomic.StoreInt32(&pv.copying, copying)
 	pv.writeMailHeaders()
 	if strings.EqualFold(pv.part.MIMEType, "text") {
 		pv.source = parse.StripAnsi(pv.hyperlinks(pv.source))
 	}
-	pv.filter.Stdin = pv.source
-	pv.filter.Stdout = pv.pagerin
-	pv.filter.Stderr = pv.pagerin
-	err := pv.filter.Start()
-	if err != nil {
-		log.Errorf("error running filter: %v", err)
-		return
+	if pv.filter != pv.pager {
+		// Filter is a separate process that needs to output to the pager.
+		pv.filter.Stdin = pv.source
+		pv.filter.Stdout = pv.pagerin
+		pv.filter.Stderr = pv.pagerin
+		err := pv.filter.Start()
+		if err != nil {
+			log.Errorf("error running filter: %v", err)
+			return
+		}
 	}
 	go func() {
 		defer log.PanicHandler()
 		defer atomic.StoreInt32(&pv.copying, 0)
-		err = pv.filter.Wait()
-		if err != nil {
-			log.Errorf("error waiting for filter: %v", err)
-			return
+		var err error
+		if pv.filter == pv.pager {
+			// Filter already implements its own paging.
+			_, err = io.Copy(pv.pagerin, pv.source)
+			if err != nil {
+				log.Errorf("io.Copy: %s", err)
+			}
+		} else {
+			err = pv.filter.Wait()
+			if err != nil {
+				log.Errorf("filter.Wait: %v", err)
+			}
 		}
 		err = pv.pagerin.Close()
 		if err != nil {
 			log.Errorf("error closing pager pipe: %v", err)
-			return
 		}
 	}()
 }
 
 func (pv *PartViewer) writeMailHeaders() {
 	info := pv.msg.MessageInfo()
-	if config.Viewer.ShowHeaders && info.RFC822Headers != nil {
-		var file io.WriteCloser
+	if !config.Viewer.ShowHeaders || info.RFC822Headers == nil {
+		return
+	}
+	if pv.filter == pv.pager {
+		// Filter already implements its own paging.
+		// Piping another filter into it will cause mayhem.
+		return
+	}
+	var file io.WriteCloser
 
-		for _, f := range config.Filters {
-			if f.Type != config.FILTER_HEADERS {
-				continue
-			}
-			log.Debugf("<%s> piping headers in filter: %s",
-				info.Envelope.MessageId, f.Command)
-			filter := exec.Command("sh", "-c", f.Command)
-			if pv.filter != nil {
-				// inherit from filter env
-				filter.Env = pv.filter.Env
-			}
+	for _, f := range config.Filters {
+		if f.Type != config.FILTER_HEADERS {
+			continue
+		}
+		log.Debugf("<%s> piping headers in filter: %s",
+			info.Envelope.MessageId, f.Command)
+		filter := exec.Command("sh", "-c", f.Command)
+		if pv.filter != nil {
+			// inherit from filter env
+			filter.Env = pv.filter.Env
+		}
 
-			stdin, err := filter.StdinPipe()
+		stdin, err := filter.StdinPipe()
+		if err == nil {
+			filter.Stdout = pv.pagerin
+			filter.Stderr = pv.pagerin
+			err := filter.Start()
 			if err == nil {
-				filter.Stdout = pv.pagerin
-				filter.Stderr = pv.pagerin
-				err := filter.Start()
-				if err == nil {
-					//nolint:errcheck // who cares?
-					defer filter.Wait()
-					file = stdin
-				} else {
-					log.Errorf(
-						"failed to start header filter: %v",
-						err)
-				}
+				//nolint:errcheck // who cares?
+				defer filter.Wait()
+				file = stdin
 			} else {
-				log.Errorf("failed to create pipe: %v", err)
+				log.Errorf(
+					"failed to start header filter: %v",
+					err)
 			}
-			break
-		}
-		if file == nil {
-			file = pv.pagerin
 		} else {
-			defer file.Close()
+			log.Errorf("failed to create pipe: %v", err)
 		}
+		break
+	}
 
-		var buf bytes.Buffer
-		err := textproto.WriteHeader(&buf, info.RFC822Headers.Header.Header)
-		if err != nil {
-			log.Errorf("failed to format headers: %v", err)
-		}
-		_, err = file.Write(bytes.TrimRight(buf.Bytes(), "\r\n"))
-		if err != nil {
-			log.Errorf("failed to write headers: %v", err)
-		}
+	if file == nil {
+		file = pv.pagerin
+	} else {
+		defer file.Close()
+	}
 
-		// virtual header
-		if len(info.Labels) != 0 {
-			labels := fmtHeader(info, "Labels", "", "", "", "")
-			_, err := file.Write([]byte(fmt.Sprintf("\r\nLabels: %s", labels)))
-			if err != nil {
-				log.Errorf("failed to write to labels: %v", err)
-			}
-		}
-		_, err = file.Write([]byte{'\r', '\n', '\r', '\n'})
+	var buf bytes.Buffer
+	err := textproto.WriteHeader(&buf, info.RFC822Headers.Header.Header)
+	if err != nil {
+		log.Errorf("failed to format headers: %v", err)
+	}
+	_, err = file.Write(bytes.TrimRight(buf.Bytes(), "\r\n"))
+	if err != nil {
+		log.Errorf("failed to write headers: %v", err)
+	}
+
+	// virtual header
+	if len(info.Labels) != 0 {
+		labels := fmtHeader(info, "Labels", "", "", "", "")
+		_, err := file.Write([]byte(fmt.Sprintf("\r\nLabels: %s", labels)))
 		if err != nil {
-			log.Errorf("failed to write empty line: %v", err)
+			log.Errorf("failed to write to labels: %v", err)
 		}
 	}
+	_, err = file.Write([]byte{'\r', '\n', '\r', '\n'})
+	if err != nil {
+		log.Errorf("failed to write empty line: %v", err)
+	}
 }
 
 func (pv *PartViewer) hyperlinks(r io.Reader) (reader io.Reader) {
@@ -711,10 +789,20 @@ func (pv *PartViewer) Invalidate() {
 
 func (pv *PartViewer) Draw(ctx *ui.Context) {
 	style := pv.uiConfig.GetStyle(config.STYLE_DEFAULT)
-	if pv.filter == nil {
+	switch {
+	case pv.filter == nil && canInline(pv.part.FullMIMEType()) && pv.err == nil:
+		pv.inlineImg = true
+	case pv.filter == nil:
+		// No filter, can't inline, and/or we attempted to inline an image
+		// and resulted in an error (maybe because of a bad encoding or
+		// the terminal doesn't support any graphics protocol).
 		ctx.Fill(0, 0, ctx.Width(), ctx.Height(), ' ', style)
 		pv.noFilter.Draw(ctx)
 		return
+	case !pv.fetched:
+		w, h := ctx.Window().Size()
+		pv.filter.Env = append(pv.filter.Env, fmt.Sprintf("COLUMNS=%d", w))
+		pv.filter.Env = append(pv.filter.Env, fmt.Sprintf("LINES=%d", h))
 	}
 	if !pv.fetched {
 		pv.msg.FetchBodyPart(pv.index, pv.SetSource)
@@ -728,15 +816,51 @@ func (pv *PartViewer) Draw(ctx *ui.Context) {
 	if pv.term != nil {
 		pv.term.Draw(ctx)
 	}
+	if pv.image != nil && (pv.resized(ctx) || pv.graphic == nil) {
+		// This path should only occur on resizes or the first pass
+		// after the image is downloaded and could be slow due to
+		// encoding the image to either sixel or uploading via the kitty
+		// protocol. Generally it's pretty fast since we will only ever
+		// be downsizing images
+		vx := ctx.Window().Vx
+		if pv.graphic == nil {
+			var err error
+			pv.graphic, err = vx.NewImage(pv.image)
+			if err != nil {
+				log.Errorf("Couldn't create image: %v", err)
+				return
+			}
+		}
+		pv.graphic.Resize(pv.width, pv.height)
+	}
+	if pv.graphic != nil {
+		w, h := pv.graphic.CellSize()
+		win := align.Center(ctx.Window(), w, h)
+		pv.graphic.Draw(win)
+	}
 }
 
 func (pv *PartViewer) Cleanup() {
 	if pv.term != nil {
 		pv.term.Close()
 	}
+	if pv.graphic != nil {
+		pv.graphic.Destroy()
+	}
 }
 
-func (pv *PartViewer) Event(event tcell.Event) bool {
+func (pv *PartViewer) resized(ctx *ui.Context) bool {
+	w := ctx.Width()
+	h := ctx.Height()
+	if pv.width != w || pv.height != h {
+		pv.width = w
+		pv.height = h
+		return true
+	}
+	return false
+}
+
+func (pv *PartViewer) Event(event vaxis.Event) bool {
 	if pv.term != nil {
 		return pv.term.Event(event)
 	}
@@ -779,3 +903,12 @@ func (hv *HeaderView) Draw(ctx *ui.Context) {
 func (hv *HeaderView) Invalidate() {
 	ui.Invalidate()
 }
+
+func canInline(mime string) bool {
+	for _, ext := range supportedImageTypes {
+		if mime == ext {
+			return true
+		}
+	}
+	return false
+}
