@@ -15,8 +15,12 @@ func init() {
 	commands.Register(Connection{})
 }
 
+func (Connection) Description() string {
+	return "Disconnect or reconnect the current account."
+}
+
 func (Connection) Context() commands.CommandContext {
-	return commands.ACCOUNT
+	return commands.MESSAGE_LIST
 }
 
 func (Connection) Aliases() []string {
