@@ -11,8 +11,12 @@ func init() {
 	commands.Register(Encrypt{})
 }
 
+func (Encrypt) Description() string {
+	return "Toggle encryption of the message to all recipients."
+}
+
 func (Encrypt) Context() commands.CommandContext {
-	return commands.COMPOSE
+	return commands.COMPOSE_EDIT | commands.COMPOSE_REVIEW
 }
 
 func (Encrypt) Aliases() []string {
