@@ -1,7 +1,6 @@
 package compose
 
 import (
-	"bytes"
 	"fmt"
 
 	"git.sr.ht/~rjarry/aerc/app"
@@ -10,16 +9,20 @@ import (
 )
 
 type Multipart struct {
-	Remove bool   `opt:"-d"`
-	Mime   string `opt:"mime" metavar:"<mime/type>" complete:"CompleteMime"`
+	Remove bool   `opt:"-d" desc:"Remove the specified mime/type."`
+	Mime   string `opt:"mime" metavar:"<mime/type>" complete:"CompleteMime" desc:"MIME/type name."`
 }
 
 func init() {
 	commands.Register(Multipart{})
 }
 
+func (Multipart) Description() string {
+	return "Convert the message to multipart with the given mime/type part."
+}
+
 func (Multipart) Context() commands.CommandContext {
-	return commands.COMPOSE
+	return commands.COMPOSE_EDIT | commands.COMPOSE_REVIEW
 }
 
 func (Multipart) Aliases() []string {
@@ -52,7 +55,7 @@ func (m Multipart) Execute(args []string) error {
 			map[string]string{"Charset": "UTF-8"},
 			// the actual content of the part will be rendered
 			// every time the body of the email is updated
-			bytes.NewReader([]byte{}),
+			nil,
 		)
 		if err != nil {
 			return err
