@@ -8,21 +8,25 @@ import (
 
 	"git.sr.ht/~rjarry/aerc/app"
 	"git.sr.ht/~rjarry/aerc/commands"
-	"git.sr.ht/~rjarry/aerc/log"
+	"git.sr.ht/~rjarry/aerc/lib/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
 type Postpone struct {
-	Folder string `opt:"-t" complete:"CompleteFolder"`
+	Folder string `opt:"-t" complete:"CompleteFolder" desc:"Override the target folder."`
 }
 
 func init() {
 	commands.Register(Postpone{})
 }
 
+func (Postpone) Description() string {
+	return "Save the current state of the message to the postpone folder."
+}
+
 func (Postpone) Context() commands.CommandContext {
-	return commands.COMPOSE
+	return commands.COMPOSE_REVIEW
 }
 
 func (Postpone) Aliases() []string {
