@@ -7,7 +7,7 @@ import (
 )
 
 type Help struct {
-	Topic string `opt:"topic" action:"ParseTopic" default:"aerc" complete:"CompleteTopic"`
+	Topic string `opt:"topic" action:"ParseTopic" default:"aerc" complete:"CompleteTopic" desc:"Help topic."`
 }
 
 var pages = []string{
@@ -32,6 +32,10 @@ func init() {
 	Register(Help{})
 }
 
+func (Help) Description() string {
+	return "Display one of aerc's man pages in the embedded terminal."
+}
+
 func (Help) Context() CommandContext {
 	return GLOBAL
 }
