@@ -12,22 +12,26 @@ import (
 	"git.sr.ht/~rjarry/aerc/lib/log"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
 	"git.sr.ht/~rjarry/aerc/models"
-	"git.sr.ht/~rjarry/go-opt"
+	"git.sr.ht/~rjarry/go-opt/v2"
 )
 
 type Menu struct {
-	ErrExit     bool   `opt:"-e"`
-	Background  bool   `opt:"-b"`
-	Accounts    bool   `opt:"-a"`
-	Directories bool   `opt:"-d"`
-	Command     string `opt:"-c"`
-	Xargs       string `opt:"..." complete:"CompleteXargs"`
+	ErrExit     bool   `opt:"-e" desc:"Stop executing commands on the first error."`
+	Background  bool   `opt:"-b" desc:"Do NOT spawn the popover dialog."`
+	Accounts    bool   `opt:"-a" desc:"Feed command with account names."`
+	Directories bool   `opt:"-d" desc:"Feed command with folder names."`
+	Command     string `opt:"-c" desc:"Override [general].default-menu-cmd."`
+	Xargs       string `opt:"..." complete:"CompleteXargs" desc:"Command name."`
 }
 
 func init() {
 	Register(Menu{})
 }
 
+func (Menu) Description() string {
+	return "Open a popover dialog."
+}
+
 func (Menu) Context() CommandContext {
 	return GLOBAL
 }
