@@ -8,22 +8,30 @@ import (
 	"git.sr.ht/~rjarry/aerc/app"
 	"git.sr.ht/~rjarry/aerc/commands"
 	"git.sr.ht/~rjarry/aerc/lib"
+	cryptoutil "git.sr.ht/~rjarry/aerc/lib/crypto/util"
 	"git.sr.ht/~rjarry/aerc/lib/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
+	"github.com/emersion/go-message/mail"
+	"github.com/pkg/errors"
 )
 
 type Copy struct {
-	CreateFolders     bool                     `opt:"-p"`
-	Account           string                   `opt:"-a" complete:"CompleteAccount"`
-	MultiFileStrategy *types.MultiFileStrategy `opt:"-m" action:"ParseMFS" complete:"CompleteMFS"`
-	Folder            string                   `opt:"folder" complete:"CompleteFolder"`
+	CreateFolders     bool                     `opt:"-p" desc:"Create folder if it does not exist."`
+	Decrypt           bool                     `opt:"-d" desc:"Decrypt the message before copying."`
+	Account           string                   `opt:"-a" complete:"CompleteAccount" desc:"Copy to the specified account."`
+	MultiFileStrategy *types.MultiFileStrategy `opt:"-m" action:"ParseMFS" complete:"CompleteMFS" desc:"Multi-file strategy."`
+	Folder            string                   `opt:"folder" complete:"CompleteFolder" desc:"Target folder."`
 }
 
 func init() {
 	commands.Register(Copy{})
 }
 
+func (Copy) Description() string {
+	return "Copy the selected message(s) to the specified folder."
+}
+
 func (Copy) Context() commands.CommandContext {
 	return commands.MESSAGE_LIST | commands.MESSAGE_VIEWER
 }
@@ -75,6 +83,16 @@ func (c Copy) Execute(args []string) error {
 		return err
 	}
 
+	// when the decrypt flag is set, add the current account to c.Account to
+	// ensure that we do not take the store.Copy route.
+	if c.Decrypt {
+		if acct := app.SelectedAccount(); acct != nil {
+			c.Account = acct.Name()
+		} else {
+			return errors.New("no account name found")
+		}
+	}
+
 	if len(c.Account) == 0 {
 		store.Copy(uids, c.Folder, c.CreateFolders, c.MultiFileStrategy,
 			func(msg types.WorkerMessage) {
@@ -97,6 +115,24 @@ func (c Copy) Execute(args []string) error {
 	var messages []*types.FullMessage
 	fetchDone := make(chan bool, 1)
 	store.FetchFull(uids, func(fm *types.FullMessage) {
+		if fm == nil {
+			return
+		}
+
+		if c.Decrypt {
+			h := new(mail.Header)
+			msg, ok := store.Messages[fm.Content.Uid]
+			if ok {
+				h = msg.RFC822Headers
+			}
+			cleartext, err := cryptoutil.Cleartext(fm.Content.Reader, *h)
+			if err != nil {
+				log.Debugf("could not decrypt message %v", fm.Content.Uid)
+			} else {
+				fm.Content.Reader = bytes.NewReader(cleartext)
+			}
+		}
+
 		messages = append(messages, fm)
 		if len(messages) == len(uids) {
 			fetchDone <- true
@@ -143,7 +179,7 @@ func (c Copy) Execute(args []string) error {
 	return nil
 }
 
-func (c Copy) CallBack(msg types.WorkerMessage, uids []uint32, store *lib.MessageStore) {
+func (c Copy) CallBack(msg types.WorkerMessage, uids []models.UID, store *lib.MessageStore) {
 	dest := c.Folder
 	if len(c.Account) != 0 {
 		dest = fmt.Sprintf("%s in %s", c.Folder, c.Account)
