@@ -13,21 +13,42 @@ import (
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
-type Delete struct{}
+type Delete struct {
+	MultiFileStrategy *types.MultiFileStrategy `opt:"-m" action:"ParseMFS" complete:"CompleteMFS" desc:"Multi-file strategy."`
+}
 
 func init() {
 	commands.Register(Delete{})
 }
 
+func (Delete) Description() string {
+	return "Delete the selected message(s)."
+}
+
 func (Delete) Context() commands.CommandContext {
-	return commands.MESSAGE
+	return commands.MESSAGE_LIST | commands.MESSAGE_VIEWER
 }
 
 func (Delete) Aliases() []string {
 	return []string{"delete", "delete-message"}
 }
 
-func (Delete) Execute(args []string) error {
+func (d *Delete) ParseMFS(arg string) error {
+	if arg != "" {
+		mfs, ok := types.StrToStrategy[arg]
+		if !ok {
+			return fmt.Errorf("invalid multi-file strategy %s", arg)
+		}
+		d.MultiFileStrategy = &mfs
+	}
+	return nil
+}
+
+func (Delete) CompleteMFS(arg string) []string {
+	return commands.FilterList(types.StrategyStrs(), arg, nil)
+}
+
+func (d Delete) Execute(args []string) error {
 	h := newHelper()
 	store, err := h.store()
 	if err != nil {
@@ -46,7 +67,7 @@ func (Delete) Execute(args []string) error {
 	marker.ClearVisualMark()
 	// caution, can be nil
 	next := findNextNonDeleted(uids, store)
-	store.Delete(uids, func(msg types.WorkerMessage) {
+	store.Delete(uids, d.MultiFileStrategy, func(msg types.WorkerMessage) {
 		switch msg := msg.(type) {
 		case *types.Done:
 			var s string
@@ -75,7 +96,11 @@ func (Delete) Execute(args []string) error {
 								app.PushError(err.Error())
 								return
 							}
-							nextMv := app.NewMessageViewer(acct, view)
+							nextMv, err := app.NewMessageViewer(acct, view)
+							if err != nil {
+								app.PushError(err.Error())
+								return
+							}
 							app.ReplaceTab(mv, nextMv, next.Envelope.Subject, true)
 						})
 				}
@@ -100,7 +125,7 @@ func (Delete) Execute(args []string) error {
 	return nil
 }
 
-func findNextNonDeleted(deleted []uint32, store *lib.MessageStore) *models.MessageInfo {
+func findNextNonDeleted(deleted []models.UID, store *lib.MessageStore) *models.MessageInfo {
 	var next, previous *models.MessageInfo
 	stepper := []func(){store.Next, store.Prev}
 	for _, stepFn := range stepper {
@@ -129,7 +154,7 @@ func findNextNonDeleted(deleted []uint32, store *lib.MessageStore) *models.Messa
 	return next
 }
 
-func contains(uids []uint32, uid uint32) bool {
+func contains(uids []models.UID, uid models.UID) bool {
 	for _, item := range uids {
 		if item == uid {
 			return true
