@@ -4,20 +4,25 @@ import (
 	"fmt"
 
 	"git.sr.ht/~rjarry/aerc/commands"
+	"git.sr.ht/~rjarry/aerc/models"
 )
 
 type Mark struct {
-	All         bool `opt:"-a" aliases:"mark,unmark"`
-	Toggle      bool `opt:"-t" aliases:"mark,unmark"`
-	Visual      bool `opt:"-v" aliases:"mark,unmark"`
-	VisualClear bool `opt:"-V" aliases:"mark,unmark"`
-	Thread      bool `opt:"-T" aliases:"mark,unmark"`
+	All         bool `opt:"-a" aliases:"mark,unmark" desc:"Mark all messages in current folder."`
+	Toggle      bool `opt:"-t" aliases:"mark,unmark" desc:"Toggle the marked state."`
+	Visual      bool `opt:"-v" aliases:"mark,unmark" desc:"Enter / leave visual mark mode."`
+	VisualClear bool `opt:"-V" aliases:"mark,unmark" desc:"Same as -v but does not clear existing selection."`
+	Thread      bool `opt:"-T" aliases:"mark,unmark" desc:"Mark all messages from the selected thread."`
 }
 
 func init() {
 	commands.Register(Mark{})
 }
 
+func (Mark) Description() string {
+	return "Mark, unmark or remark messages."
+}
+
 func (Mark) Context() commands.CommandContext {
 	return commands.MESSAGE_LIST | commands.MESSAGE_VIEWER
 }
@@ -28,7 +33,7 @@ func (Mark) Aliases() []string {
 
 func (m Mark) Execute(args []string) error {
 	h := newHelper()
-	OnSelectedMessage := func(fn func(uint32)) error {
+	OnSelectedMessage := func(fn func(models.UID)) error {
 		if fn == nil {
 			return fmt.Errorf("no operation selected")
 		}
@@ -58,7 +63,7 @@ func (m Mark) Execute(args []string) error {
 
 	switch args[0] {
 	case "mark":
-		var modFunc func(uint32)
+		var modFunc func(models.UID)
 		if m.Toggle {
 			modFunc = marker.ToggleMark
 		} else {
