@@ -9,15 +9,19 @@ import (
 )
 
 type ModifyLabels struct {
-	Labels []string `opt:"..." metavar:"[+-]<label>" complete:"CompleteLabels"`
+	Labels []string `opt:"..." metavar:"[+-]<label>" complete:"CompleteLabels" desc:"Message label."`
 }
 
 func init() {
 	commands.Register(ModifyLabels{})
 }
 
+func (ModifyLabels) Description() string {
+	return "Modify message labels."
+}
+
 func (ModifyLabels) Context() commands.CommandContext {
-	return commands.MESSAGE
+	return commands.MESSAGE_LIST | commands.MESSAGE_VIEWER
 }
 
 func (ModifyLabels) Aliases() []string {
