@@ -1,6 +1,7 @@
 package msg
 
 import (
+	"bytes"
 	"errors"
 	"fmt"
 	"io"
@@ -12,15 +13,19 @@ import (
 
 	"git.sr.ht/~rjarry/aerc/app"
 	"git.sr.ht/~rjarry/aerc/commands"
-	"git.sr.ht/~rjarry/aerc/log"
+	cryptoutil "git.sr.ht/~rjarry/aerc/lib/crypto/util"
+	"git.sr.ht/~rjarry/aerc/lib/log"
+	"git.sr.ht/~rjarry/aerc/models"
 	mboxer "git.sr.ht/~rjarry/aerc/worker/mbox"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
 type Pipe struct {
-	Background bool   `opt:"-b"`
-	Full       bool   `opt:"-m"`
-	Part       bool   `opt:"-p"`
+	Background bool   `opt:"-b" desc:"Run the command in the background."`
+	Silent     bool   `opt:"-s" desc:"Silently close the terminal tab after the command exits."`
+	Full       bool   `opt:"-m" desc:"Pipe the full message."`
+	Decrypt    bool   `opt:"-d" desc:"Decrypt the full message before piping."`
+	Part       bool   `opt:"-p" desc:"Only pipe the selected message part."`
 	Command    string `opt:"..."`
 }
 
@@ -28,8 +33,12 @@ func init() {
 	commands.Register(Pipe{})
 }
 
+func (Pipe) Description() string {
+	return "Pipe the selected message(s) into the given shell command."
+}
+
 func (Pipe) Context() commands.CommandContext {
-	return commands.MESSAGE
+	return commands.MESSAGE_LIST | commands.MESSAGE_VIEWER
 }
 
 func (Pipe) Aliases() []string {
@@ -41,6 +50,10 @@ func (p Pipe) Execute(args []string) error {
 }
 
 func (p Pipe) Run(cb func()) error {
+	if p.Decrypt {
+		// Decrypt implies fetching the full message
+		p.Full = true
+	}
 	if p.Full && p.Part {
 		return errors.New("-m and -p are mutually exclusive")
 	}
@@ -60,7 +73,7 @@ func (p Pipe) Run(cb func()) error {
 
 	doTerm := func(reader io.Reader, name string) {
 		cmd := []string{"sh", "-c", p.Command}
-		term, err := commands.QuickTerm(cmd, reader)
+		term, err := commands.QuickTerm(cmd, reader, p.Silent)
 		if err != nil {
 			app.PushError(err.Error())
 			return
@@ -114,7 +127,7 @@ func (p Pipe) Run(cb func()) error {
 	app.PushStatus("Fetching messages ...", 10*time.Second)
 
 	if p.Full {
-		var uids []uint32
+		var uids []models.UID
 		var title string
 
 		h := newHelper()
@@ -153,11 +166,41 @@ func (p Pipe) Run(cb func()) error {
 		}
 
 		var messages []*types.FullMessage
+		var errors []error
 		done := make(chan bool, 1)
 
 		store.FetchFull(uids, func(fm *types.FullMessage) {
-			messages = append(messages, fm)
-			if len(messages) == len(uids) {
+			if p.Decrypt {
+				info := store.Messages[fm.Content.Uid]
+				if info == nil {
+					goto addMessage
+				}
+				var buf bytes.Buffer
+				cleartext, err := cryptoutil.Cleartext(
+					io.TeeReader(fm.Content.Reader, &buf),
+					info.RFC822Headers.Copy(),
+				)
+				if err != nil {
+					log.Warnf("continue encrypted: %v", err)
+					fm.Content.Reader = bytes.NewReader(buf.Bytes())
+				} else {
+					fm.Content.Reader = bytes.NewReader(cleartext)
+				}
+			}
+		addMessage:
+			info := store.Messages[fm.Content.Uid]
+			switch {
+			case info != nil && info.Envelope != nil:
+				messages = append(messages, fm)
+			case info != nil && info.Error != nil:
+				app.PushError(info.Error.Error())
+				errors = append(errors, info.Error)
+			default:
+				err := fmt.Errorf("%v nil info", fm.Content.Uid)
+				app.PushError(err.Error())
+				errors = append(errors, err)
+			}
+			if len(messages)+len(errors) == len(uids) {
 				done <- true
 			}
 		})
@@ -180,7 +223,10 @@ func (p Pipe) Run(cb func()) error {
 			is_git_patches := false
 			for _, msg := range messages {
 				info := store.Messages[msg.Content.Uid]
-				if info != nil && patchSeriesRe.MatchString(info.Envelope.Subject) {
+				if info == nil || info.Envelope == nil {
+					continue
+				}
+				if patchSeriesRe.MatchString(info.Envelope.Subject) {
 					is_git_patches = true
 					break
 				}
@@ -191,7 +237,8 @@ func (p Pipe) Run(cb func()) error {
 				sort.Slice(messages, func(i, j int) bool {
 					infoi := store.Messages[messages[i].Content.Uid]
 					infoj := store.Messages[messages[j].Content.Uid]
-					if infoi == nil || infoj == nil {
+					if infoi == nil || infoi.Envelope == nil ||
+						infoj == nil || infoj.Envelope == nil {
 						return false
 					}
 					return infoi.Envelope.Subject < infoj.Envelope.Subject
