@@ -12,16 +12,21 @@ import (
 )
 
 type FlagMsg struct {
-	Toggle   bool         `opt:"-t"`
-	Answered bool         `opt:"-a" aliases:"flag,unflag"`
-	Flag     models.Flags `opt:"-x" aliases:"flag,unflag" action:"ParseFlag" complete:"CompleteFlag"`
-	FlagName string
+	Toggle    bool         `opt:"-t" desc:"Toggle between set and unset."`
+	Answered  bool         `opt:"-a" aliases:"flag,unflag" desc:"Set/unset the answered flag."`
+	Forwarded bool         `opt:"-f" aliases:"flag,unflag" desc:"Set/unset the forwarded flag."`
+	Flag      models.Flags `opt:"-x" aliases:"flag,unflag" action:"ParseFlag" complete:"CompleteFlag" desc:"Flag name."`
+	FlagName  string
 }
 
 func init() {
 	commands.Register(FlagMsg{})
 }
 
+func (FlagMsg) Description() string {
+	return "Set or unset a flag on the marked or selected messages."
+}
+
 func (FlagMsg) Context() commands.CommandContext {
 	return commands.MESSAGE_LIST | commands.MESSAGE_VIEWER
 }
@@ -38,6 +43,9 @@ func (f *FlagMsg) ParseFlag(arg string) error {
 	case "answered":
 		f.Flag = models.AnsweredFlag
 		f.FlagName = "answered"
+	case "forwarded":
+		f.Flag = models.ForwardedFlag
+		f.FlagName = "forwarded"
 	case "flagged":
 		f.Flag = models.FlaggedFlag
 		f.FlagName = "flagged"
@@ -50,7 +58,7 @@ func (f *FlagMsg) ParseFlag(arg string) error {
 	return nil
 }
 
-var validFlags = []string{"seen", "answered", "flagged", "draft"}
+var validFlags = []string{"seen", "answered", "forwarded", "flagged", "draft"}
 
 func (*FlagMsg) CompleteFlag(arg string) []string {
 	return commands.FilterList(validFlags, arg, nil)
@@ -76,6 +84,10 @@ func (f FlagMsg) Execute(args []string) error {
 			f.Flag = models.AnsweredFlag
 			f.FlagName = "answered"
 		}
+		if f.Forwarded {
+			f.Flag = models.ForwardedFlag
+			f.FlagName = "forwarded"
+		}
 		if f.Flag == 0 {
 			f.Flag = models.FlaggedFlag
 			f.FlagName = "flagged"
@@ -89,8 +101,8 @@ func (f FlagMsg) Execute(args []string) error {
 	}
 
 	// UIDs of messages to enable or disable the flag for.
-	var toEnable []uint32
-	var toDisable []uint32
+	var toEnable []models.UID
+	var toDisable []models.UID
 
 	if f.Toggle {
 		// If toggling, split messages into those that need to
