@@ -10,7 +10,7 @@ import (
 )
 
 type Init struct {
-	Force bool   `opt:"-f"`
+	Force bool   `opt:"-f" desc:"Overwrite any existing project."`
 	Name  string `opt:"name" required:"false"`
 }
 
@@ -18,6 +18,10 @@ func init() {
 	register(Init{})
 }
 
+func (Init) Description() string {
+	return "Create a new project."
+}
+
 func (Init) Context() commands.CommandContext {
 	return commands.GLOBAL
 }
