@@ -1,20 +1,24 @@
 package commands
 
 import (
-	"git.sr.ht/~rjarry/go-opt"
+	"git.sr.ht/~rjarry/go-opt/v2"
 
 	"git.sr.ht/~rjarry/aerc/app"
 )
 
 type Prompt struct {
 	Text string   `opt:"text"`
-	Cmd  []string `opt:"..." complete:"CompleteCommand"`
+	Cmd  []string `opt:"..." complete:"CompleteCommand" desc:"Command name."`
 }
 
 func init() {
 	Register(Prompt{})
 }
 
+func (Prompt) Description() string {
+	return "Prompt for user input and execute a command."
+}
+
 func (Prompt) Context() CommandContext {
 	return GLOBAL
 }
