@@ -3,7 +3,7 @@ package commands
 import (
 	"git.sr.ht/~rjarry/aerc/app"
 	"git.sr.ht/~rjarry/aerc/config"
-	"github.com/gdamore/tcell/v2"
+	"git.sr.ht/~rockorager/vaxis"
 	"github.com/pkg/errors"
 )
 
@@ -15,6 +15,10 @@ func init() {
 	Register(SendKeys{})
 }
 
+func (SendKeys) Description() string {
+	return "Send keystrokes to the currently visible terminal."
+}
+
 func (SendKeys) Context() CommandContext {
 	return GLOBAL
 }
@@ -40,7 +44,10 @@ func (s SendKeys) Execute(args []string) error {
 	}
 
 	for _, key := range keys2send {
-		ev := tcell.NewEventKey(key.Key, key.Rune, key.Modifiers)
+		ev := vaxis.Key{
+			Keycode:   key.Key,
+			Modifiers: key.Modifiers,
+		}
 		term.Event(ev)
 	}
 
