@@ -1,6 +1,7 @@
 package config
 
 import (
+	"bytes"
 	"errors"
 	"fmt"
 	"net/url"
@@ -13,11 +14,17 @@ import (
 	"strings"
 	"time"
 
-	"git.sr.ht/~rjarry/aerc/log"
+	"git.sr.ht/~rjarry/aerc/lib/log"
 	"github.com/emersion/go-message/mail"
 	"github.com/go-ini/ini"
 )
 
+var (
+	EnablePinentry  func()
+	DisablePinentry func()
+	SetPinentryEnv  func(*exec.Cmd)
+)
+
 type RemoteConfig struct {
 	Value       string
 	PasswordCmd string
@@ -52,11 +59,26 @@ func (c *RemoteConfig) ConnectionString() (string, error) {
 	pw := c.cache
 
 	if pw == "" {
+		usePinentry := EnablePinentry != nil &&
+			DisablePinentry != nil &&
+			SetPinentryEnv != nil
+
 		cmd := exec.Command("sh", "-c", c.PasswordCmd)
 		cmd.Stdin = os.Stdin
+
+		buf := new(bytes.Buffer)
+		cmd.Stderr = buf
+
+		if usePinentry {
+			EnablePinentry()
+			defer DisablePinentry()
+			SetPinentryEnv(cmd)
+		}
+
 		output, err := cmd.Output()
 		if err != nil {
-			return "", fmt.Errorf("failed to read password: %w", err)
+			return "", fmt.Errorf("failed to read password: %v: %w",
+				buf.String(), err)
 		}
 		pw = strings.TrimSpace(string(output))
 	}
@@ -69,15 +91,19 @@ func (c *RemoteConfig) ConnectionString() (string, error) {
 }
 
 type AccountConfig struct {
-	Name string
+	Name    string
+	Backend string
 	// backend specific
 	Params map[string]string
 
 	Archive           string          `ini:"archive" default:"Archive"`
-	CopyTo            string          `ini:"copy-to"`
+	CopyTo            []string        `ini:"copy-to" delim:","`
+	CopyToReplied     bool            `ini:"copy-to-replied" default:"false"`
+	StripBcc          bool            `ini:"strip-bcc" default:"true"`
 	Default           string          `ini:"default" default:"INBOX"`
 	Postpone          string          `ini:"postpone" default:"Drafts"`
 	From              *mail.Address   `ini:"from"`
+	UseEnvelopeFrom   bool            `ini:"use-envelope-from" default:"false"`
 	Aliases           []*mail.Address `ini:"aliases"`
 	Source            string          `ini:"source" parse:"ParseSource"`
 	Folders           []string        `ini:"folders" delim:","`
@@ -107,6 +133,7 @@ type AccountConfig struct {
 	PgpAttachKey            bool   `ini:"pgp-attach-key"`
 	PgpOpportunisticEncrypt bool   `ini:"pgp-opportunistic-encrypt"`
 	PgpErrorLevel           int    `ini:"pgp-error-level" parse:"ParsePgpErrorLevel" default:"warn"`
+	PgpSelfEncrypt          bool   `ini:"pgp-self-encrypt"`
 
 	// AuthRes
 	TrustedAuthRes []string `ini:"trusted-authres" delim:","`
@@ -131,18 +158,33 @@ func parseAccountsFromFile(root string, accts []string, filename string) error {
 	}
 
 	starttls_warned := false
+	var globals *ini.Section
 	for _, _sec := range file.SectionStrings() {
 		if _sec == "DEFAULT" {
+			globals = file.Section(_sec)
 			continue
 		}
 		if len(accts) > 0 && !contains(accts, _sec) {
 			continue
 		}
 		sec := file.Section(_sec)
+		for key, val := range globals.KeysHash() {
+			if !sec.HasKey(key) {
+				_, _ = sec.NewKey(key, val)
+			}
+		}
 
 		account, err := ParseAccountConfig(_sec, sec)
 		if err != nil {
-			return err
+			log.Errorf("failed to load account [%s]: %s", _sec, err)
+			Warnings = append(Warnings, Warning{
+				Title: "accounts.conf: error",
+				Body: fmt.Sprintf(
+					"Failed to load account [%s]:\n\n%s",
+					_sec, err,
+				),
+			})
+			continue
 		}
 		if _, ok := account.Params["smtp-starttls"]; ok && !starttls_warned {
 			Warnings = append(Warnings, Warning{
@@ -162,11 +204,21 @@ If you want to disable STARTTLS, append +insecure to the schema.
 	if len(accts) > 0 {
 		// Sort accounts struct to match the specified order, if we
 		// have one
-		if len(Accounts) != len(accts) {
-			return errors.New("account(s) not found")
+		var acctnames []string
+		for _, acc := range Accounts {
+			acctnames = append(acctnames, acc.Name)
 		}
+		var sortaccts []string
+		for _, acc := range accts {
+			if contains(acctnames, acc) {
+				sortaccts = append(sortaccts, acc)
+			} else {
+				log.Errorf("account [%s] not found", acc)
+			}
+		}
+
 		idx := make(map[string]int)
-		for i, acct := range accts {
+		for i, acct := range sortaccts {
 			idx[acct] = i
 		}
 		sort.Slice(Accounts, func(i, j int) bool {
@@ -219,10 +271,12 @@ func ParseAccountConfig(name string, section *ini.Section) (*AccountConfig, erro
 		}
 	}
 	if account.Source == "" {
-		return nil, fmt.Errorf("Expected source for account %s", name)
+		return nil, fmt.Errorf("missing 'source' parameter")
 	}
+
+	account.Backend = parseBackend(account.Source)
 	if account.From == nil {
-		return nil, fmt.Errorf("Expected from for account %s", name)
+		return nil, fmt.Errorf("missing 'from' parameter")
 	}
 	if len(account.Headers) > 0 {
 		defaults := []string{
@@ -243,6 +297,23 @@ func ParseAccountConfig(name string, section *ini.Section) (*AccountConfig, erro
 	return &account, nil
 }
 
+func parseBackend(source string) string {
+	u, err := url.Parse(source)
+	if err != nil {
+		return ""
+	}
+	if strings.HasPrefix(u.Scheme, "imap") {
+		return "imap"
+	}
+	if strings.HasPrefix(u.Scheme, "maildir") {
+		return "maildir"
+	}
+	if strings.HasPrefix(u.Scheme, "jmap") {
+		return "jmap"
+	}
+	return u.Scheme
+}
+
 func (a *AccountConfig) ParseSource(sec *ini.Section, key *ini.Key) (string, error) {
 	var remote RemoteConfig
 	remote.Value = key.String()
