@@ -1,7 +1,7 @@
 package config
 
 import (
-	"git.sr.ht/~rjarry/aerc/log"
+	"git.sr.ht/~rjarry/aerc/lib/log"
 	"github.com/go-ini/ini"
 )
 
@@ -32,7 +32,7 @@ func (s *StatuslineConfig) ParseColumns(sec *ini.Section, key *ini.Key) ([]*Colu
 		_, _ = sec.NewKey("column-center", "{{.PendingKeys}}")
 	}
 	if !sec.HasKey("column-right") {
-		_, _ = sec.NewKey("column-right", "{{.TrayInfo}}")
+		_, _ = sec.NewKey("column-right", "{{.TrayInfo}} | {{cwd}}")
 	}
 	return ParseColumnDefs(key, sec)
 }
