@@ -327,12 +327,15 @@ const defaultStyleset string = `
 *.selected.fg = 15
 *.selected.bold = true
 statusline_*.dim = true
-statusline_*.bg = 8
-statusline_*.fg = 15
-*warning.fg = 3
-*success.fg = 2
-*error.fg = 1
+*warning.dim = false
+*warning.bold = true
+*warning.fg = 11
+*success.dim = false
+*success.bold = true
+*success.fg = 10
+*error.dim = false
 *error.bold = true
+*error.fg = 9
 border.fg = 12
 border.bold = true
 title.bg = 12
@@ -505,8 +508,8 @@ func (ss *StyleSet) ParseStyleSet(file *ini.File) error {
 }
 
 var (
-	styleObjRe            = regexp.MustCompile(`^([\w\*\?]+)(?:\.([\w-]+,.+?)+?)?(\.selected)?\.(\w+)$`)
-	styleHeaderPatternsRe = regexp.MustCompile(`([\w-]+),(.+?)\.`)
+	styleObjRe            = regexp.MustCompile(`^([\w\*\?]+)(\.(?:[\w-]+,.+?)+?)?(\.selected)?\.(\w+)$`)
+	styleHeaderPatternsRe = regexp.MustCompile(`([\w-]+),(~/(?:.+?)/|(?:.+?))\.`)
 )
 
 func (ss *StyleSet) parseKey(key *ini.Key, selected bool) error {
@@ -577,9 +580,12 @@ func (c *StyleConf) update(headerPatterns map[string]*StyleHeaderPattern, attr,
 
 	for _, p := range headerPatterns {
 		var pattern string
-		if strings.HasPrefix(p.RawPattern, "~") {
+		switch {
+		case strings.HasPrefix(p.RawPattern, "~/"):
+			pattern = p.RawPattern[2 : len(p.RawPattern)-1]
+		case strings.HasPrefix(p.RawPattern, "~"):
 			pattern = p.RawPattern[1:]
-		} else {
+		default:
 			pattern = "^" + regexp.QuoteMeta(p.RawPattern) + "$"
 		}
 
