@@ -66,6 +66,7 @@ var (
 
 func (d *dummyData) Account() string                 { return "work" }
 func (d *dummyData) AccountBackend() string          { return "maildir" }
+func (d *dummyData) AccountFrom() *mail.Address      { return &addr1 }
 func (d *dummyData) Signature() string               { return "" }
 func (d *dummyData) Folder() string                  { return "INBOX" }
 func (d *dummyData) To() []*mail.Address             { return []*mail.Address{&addr1} }
@@ -98,6 +99,7 @@ func (d *dummyData) IsUnread() bool                  { return false }
 func (d *dummyData) IsFlagged() bool                 { return false }
 func (d *dummyData) IsDraft() bool                   { return false }
 func (d *dummyData) IsMarked() bool                  { return false }
+func (d *dummyData) IsForwarded() bool               { return false }
 func (d *dummyData) MessageId() string               { return "123456789@foo.org" }
 func (d *dummyData) Size() int                       { return 420 }
 func (d *dummyData) OriginalText() string            { return "Blah blah blah" }
