@@ -33,6 +33,7 @@ type UIConfig struct {
 	MessageViewThisYearTimeFormat string        `ini:"message-view-this-year-time-format"`
 	PinnedTabMarker               string        "ini:\"pinned-tab-marker\" default:\"`\""
 	SidebarWidth                  int           `ini:"sidebar-width" default:"22"`
+	QuakeHeight                   int           `ini:"quake-terminal-height" default:"20"`
 	MessageListSplit              SplitParams   `ini:"message-list-split" parse:"ParseSplit"`
 	EmptyMessage                  string        `ini:"empty-message" default:"(no messages)"`
 	EmptyDirlist                  string        `ini:"empty-dirlist" default:"(no folders)"`
@@ -56,6 +57,7 @@ type UIConfig struct {
 	IconInvalid                   string        `ini:"icon-invalid" default:"[s!]"`
 	IconAttachment                string        `ini:"icon-attachment" default:"a"`
 	IconReplied                   string        `ini:"icon-replied" default:"r"`
+	IconForwarded                 string        `ini:"icon-forwarded" default:"f"`
 	IconNew                       string        `ini:"icon-new" default:"N"`
 	IconOld                       string        `ini:"icon-old" default:"O"`
 	IconDraft                     string        `ini:"icon-draft" default:"d"`
@@ -133,9 +135,13 @@ type uiContextKey struct {
 	value   string
 }
 
-var Ui = &UIConfig{
-	contextualCounts: make(map[uiContextType]int),
-	contextualCache:  make(map[uiContextKey]*UIConfig),
+var Ui = defaultUIConfig()
+
+func defaultUIConfig() *UIConfig {
+	return &UIConfig{
+		contextualCounts: make(map[uiContextType]int),
+		contextualCache:  make(map[uiContextKey]*UIConfig),
+	}
 }
 
 var uiContextualSectionRe = regexp.MustCompile(`^ui:(account|folder|subject)([~=])(.+)$`)
@@ -187,28 +193,10 @@ func parseUi(file *ini.File) error {
 		)
 	}
 
-	if err := Ui.loadStyleSet(Ui.StyleSetDirs); err != nil {
+	if err := Ui.LoadStyle(); err != nil {
 		return err
 	}
 
-	for _, contextualUi := range Ui.contextualUis {
-		if contextualUi.UiConfig.StyleSetName == "" &&
-			len(contextualUi.UiConfig.StyleSetDirs) == 0 {
-			continue // no need to do anything if nothing is overridden
-		}
-		// fill in the missing part from the base
-		if contextualUi.UiConfig.StyleSetName == "" {
-			contextualUi.UiConfig.StyleSetName = Ui.StyleSetName
-		} else if len(contextualUi.UiConfig.StyleSetDirs) == 0 {
-			contextualUi.UiConfig.StyleSetDirs = Ui.StyleSetDirs
-		}
-		// since at least one of them has changed, load the styleset
-		if err := contextualUi.UiConfig.loadStyleSet(
-			contextualUi.UiConfig.StyleSetDirs); err != nil {
-			return err
-		}
-	}
-
 	log.Debugf("aerc.conf: [ui] %#v", Ui)
 
 	return nil
@@ -308,6 +296,38 @@ func (*UIConfig) ParseCompletionMinChars(section *ini.Section, key *ini.Key) (in
 	return key.Int()
 }
 
+func (ui *UIConfig) ClearCache() {
+	for k := range ui.contextualCache {
+		delete(ui.contextualCache, k)
+	}
+}
+
+func (ui *UIConfig) LoadStyle() error {
+	if err := ui.loadStyleSet(ui.StyleSetDirs); err != nil {
+		return err
+	}
+
+	for _, contextualUi := range ui.contextualUis {
+		if contextualUi.UiConfig.StyleSetName == "" &&
+			len(contextualUi.UiConfig.StyleSetDirs) == 0 {
+			continue // no need to do anything if nothing is overridden
+		}
+		// fill in the missing part from the base
+		if contextualUi.UiConfig.StyleSetName == "" {
+			contextualUi.UiConfig.StyleSetName = ui.StyleSetName
+		} else if len(contextualUi.UiConfig.StyleSetDirs) == 0 {
+			contextualUi.UiConfig.StyleSetDirs = ui.StyleSetDirs
+		}
+		// since at least one of them has changed, load the styleset
+		if err := contextualUi.UiConfig.loadStyleSet(
+			contextualUi.UiConfig.StyleSetDirs); err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
 func (ui *UIConfig) loadStyleSet(styleSetDirs []string) error {
 	ui.style = NewStyleSet()
 	err := ui.style.LoadStyleSet(ui.StyleSetName, styleSetDirs)
