@@ -1,9 +1,10 @@
 /* SPDX-License-Identifier: MIT */
 /* Copyright (c) 2023 Robin Jarry */
 
-#define _XOPEN_SOURCE
+#define _XOPEN_SOURCE 700
 #include <errno.h>
 #include <getopt.h>
+#include <langinfo.h>
 #include <locale.h>
 #include <regex.h>
 #include <stdbool.h>
@@ -14,6 +15,10 @@
 #include <wchar.h>
 #include <wctype.h>
 
+#ifdef __APPLE__
+#include <xlocale.h>
+#endif
+
 static void usage(void)
 {
 	puts("usage: wrap [-h] [-w INT] [-r] [-l INT] [-f FILE]");
@@ -25,7 +30,7 @@ static void usage(void)
 	puts("  -w INT   preferred wrap margin (default 80)");
 	puts("  -r       reflow all paragraphs even if no trailing space");
 	puts("  -l INT   minimum percentage of letters in a line to be");
-	puts("           considered a paragaph");
+	puts("           considered a paragraph");
 	puts("  -f FILE  read from filename (default stdin)");
 }
 
@@ -204,6 +209,49 @@ static size_t list_item_offset(const wchar_t *buf)
 	return i;
 }
 
+static bool is_cjk(wchar_t c, bool include_syllables) {
+	/* CJK Radicals Supplement */
+	if (c >= 0x2e80 && c <= 0x2fd5)
+		return true;
+	/* CJK Compatibility */
+	if (c >= 0x3300 && c <= 0x33ff)
+		return true;
+	/* CJK Unified Ideographs Extension A */
+	if (c >= 0x3400 && c <= 0x4db5)
+		return true;
+	/* CJK Unified Ideographs */
+	if (c >= 0x4e00 && c <= 0x9fcb)
+		return true;
+	/* CJK Compatibility Ideographs */
+	if (c >= 0xf900 && c <= 0xfa6a)
+		return true;
+	/* Hangul Jamo */
+	if (c >= 0x1100 && c <= 0x11ff)
+		return true;
+	/* Hangul Compatibility Jamo */
+	if (c >= 0x3130 && c <= 0x318f)
+		return true;
+	/* Hangul Jamo Extended-A */
+	if (c >= 0xa960 && c <= 0xa97f)
+		return true;
+	/* Hangul Jamo Extended-B */
+	if (c >= 0xd7b0 && c <= 0xd7ff)
+		return true;
+
+	if (include_syllables) {
+		/* Japanese Hiragana */
+		if (c >= 0x3040 && c <= 0x309f)
+			return true;
+		/* Japanese Katakana */
+		if (c >= 0x30a0 && c <= 0x30ff)
+			return true;
+		/* Hangul Syllables */
+		if (c >= 0xac00 && c <= 0xd7af)
+			return true;
+	}
+	return false;
+}
+
 static struct paragraph *parse_line(const wchar_t *buf)
 {
 	size_t i, q, t, e, letters, indent_len, text_len;
@@ -246,7 +294,8 @@ static struct paragraph *parse_line(const wchar_t *buf)
 	e = t;
 	letters = 0;
 	while (buf[e] != L'\0') {
-		if (iswalpha((wint_t)buf[e++])) {
+		wchar_t c = buf[e++];
+		if (iswalpha((wint_t)c) || is_cjk(c, true)) {
 			letters++;
 		}
 	}
@@ -334,10 +383,24 @@ static void join_paragraph(
 
 /*
  * BUFSIZ has different values depending on the libc implementation.
- * Use a self defined value to have consistent behaviour accross all platforms.
+ * Use a self defined value to have consistent behaviour across all platforms.
  */
 #define BUFFER_SIZE 8192
 
+/*
+ * Check if a line can be split at the given character point.
+ */
+static bool is_split_point(const wchar_t c)
+{
+	if (iswspace((wint_t)c))
+		return true;
+
+	if (is_cjk(c, false))
+		return true;
+
+	return false;
+}
+
 /*
  * Write a paragraph, wrapping at words boundaries.
  *
@@ -351,6 +414,7 @@ static void write_paragraph(struct paragraph *p)
 	const wchar_t *indent = L"";
 	wchar_t *text = p->text;
 	bool more = true;
+	int wchar_count;
 	wchar_t *line;
 	size_t width;
 
@@ -360,6 +424,7 @@ static void write_paragraph(struct paragraph *p)
 		if (width + remain <= margin || p->prose_ratio < prose_ratio) {
 			/* whole paragraph fits on a single line */
 			line = text;
+			wchar_count = (int)wcslen(text);
 			more = false;
 		} else {
 			/* find split point, preferably before margin */
@@ -370,32 +435,32 @@ static void write_paragraph(struct paragraph *p)
 				if (width + w > margin && split != SIZE_MAX) {
 					break;
 				}
-				if (iswspace((wint_t)text[i])) {
+				if (is_split_point(text[i])) {
 					split = i;
 				}
 			}
 			if (split == SIZE_MAX) {
 				/* no space found to split, print a long line */
 				line = text;
+				wchar_count = (int)wcslen(text);
 				more = false;
 			} else {
-				text[split] = L'\0';
+				wchar_count = (int)split;
 				line = text;
-				split++;
 				/* find start of next word */
 				while (iswspace((wint_t)text[split])) {
 					split++;
 				}
 				if (text[split] != L'\0') {
+					remain -= (size_t)wcswidth(text, split);
 					text = &text[split];
-					remain -= split;
 				} else {
 					/* only trailing whitespace, we're done */
 					more = false;
 				}
 			}
 		}
-		wprintf(L"%ls%ls%ls\n", p->quotes, indent, line);
+		wprintf(L"%ls%ls%.*ls\n", p->quotes, indent, wchar_count, line);
 		indent = p->indent;
 	}
 }
@@ -430,7 +495,7 @@ static int set_stdio_encoding(void)
 
 	if (!locale) {
 		/* Neither LC_ALL nor LANG env vars are defined or are set to
-		 * a non existant/installed locale. Try with a generic UTF-8
+		 * a non existent/installed locale. Try with a generic UTF-8
 		 * locale which is expected to be available on all POSIX
 		 * systems. */
 		locale = setlocale(LC_ALL, "C.UTF-8");
@@ -446,7 +511,10 @@ static int set_stdio_encoding(void)
 	}
 
 	/* aerc will always send UTF-8 text, ensure that we read that properly */
-	if (!strstr(locale, "UTF-8") && !strstr(locale, "utf-8")) {
+	locale_t loc = newlocale(LC_ALL_MASK, locale, NULL);
+	char *codeset = nl_langinfo_l(CODESET, loc);
+	freelocale(loc);
+	if (!strstr(codeset, "UTF-8")) {
 		fprintf(stderr, "error: locale '%s' is not UTF-8\n", locale);
 		return 1;
 	}
