@@ -2,6 +2,7 @@ package gpgbin
 
 import (
 	"bytes"
+	"errors"
 	"io"
 
 	"git.sr.ht/~rjarry/aerc/models"
@@ -18,19 +19,17 @@ func Decrypt(r io.Reader) (*models.MessageDetails, error) {
 	args := []string{"--decrypt"}
 	g := newGpg(bytes.NewReader(orig), args)
 	_ = g.cmd.Run()
-	outRdr := bytes.NewReader(g.stdout.Bytes())
 	// Always parse stdout, even if there was an error running command.
 	// We'll find the error in the parsing
-	err = parse(outRdr, md)
-	if err != nil {
-		err = parseError(g.stderr.String())
-		switch GPGErrors[err.Error()] {
-		case ERROR_NO_PGP_DATA_FOUND:
-			md.Body = bytes.NewReader(orig)
-			return md, nil
-		default:
-			return nil, err
-		}
+	err = parseStatusFd(bytes.NewReader(g.stderr.Bytes()), md)
+
+	if errors.Is(err, NoValidOpenPgpData) {
+		md.Body = bytes.NewReader(orig)
+		return md, nil
+	} else if err != nil {
+		return nil, err
 	}
+
+	md.Body = bytes.NewReader(g.stdout.Bytes())
 	return md, nil
 }
