@@ -8,6 +8,7 @@ import (
 	_ "github.com/emersion/go-message/charset"
 
 	"git.sr.ht/~rjarry/aerc/lib/crypto"
+	"git.sr.ht/~rjarry/aerc/lib/log"
 	"git.sr.ht/~rjarry/aerc/lib/rfc822"
 	"git.sr.ht/~rjarry/aerc/models"
 )
@@ -19,8 +20,8 @@ func (fm *EmlMessage) NewReader() (io.ReadCloser, error) {
 	return io.NopCloser(bytes.NewReader(*fm)), nil
 }
 
-func (fm *EmlMessage) UID() uint32 {
-	return 0xFFFFFFF
+func (fm *EmlMessage) UID() models.UID {
+	return ""
 }
 
 func (fm *EmlMessage) Labels() ([]string, error) {
@@ -71,7 +72,10 @@ func NewEmlMessageView(full []byte, pgp crypto.Provider,
 		return
 	}
 	bs, err := rfc822.ParseEntityStructure(entity)
-	if err != nil {
+	if rfc822.IsMultipartError(err) {
+		log.Warnf("EmlView: %v", err)
+		bs = rfc822.CreateTextPlainBody()
+	} else if err != nil {
 		cb(nil, err)
 		return
 	}
