@@ -1,6 +1,7 @@
 package hooks
 
 import (
+	"bytes"
 	"os"
 	"os/exec"
 
@@ -16,7 +17,15 @@ func RunHook(h HookType) error {
 	log.Debugf("hooks: running command %q (env %v)", cmd, env)
 
 	proc := exec.Command("sh", "-c", cmd)
+	var outb, errb bytes.Buffer
+	proc.Stdout = &outb
+	proc.Stderr = &errb
 	proc.Env = os.Environ()
 	proc.Env = append(proc.Env, env...)
-	return proc.Run()
+	err := proc.Run()
+	log.Tracef("hooks: %q stdout: %s", cmd, outb.String())
+	if err != nil {
+		log.Errorf("hooks:%q stderr: %s", cmd, errb.String())
+	}
+	return err
 }
