@@ -5,17 +5,12 @@ import (
 	"context"
 	"errors"
 	"net"
-	"net/url"
 	"os"
-	"strings"
 	"sync/atomic"
 	"time"
 
-	"git.sr.ht/~rjarry/go-opt"
-
-	"git.sr.ht/~rjarry/aerc/config"
+	"git.sr.ht/~rjarry/aerc/lib/log"
 	"git.sr.ht/~rjarry/aerc/lib/xdg"
-	"git.sr.ht/~rjarry/aerc/log"
 )
 
 type AercServer struct {
@@ -27,7 +22,7 @@ type AercServer struct {
 func StartServer(handler Handler, startup context.Context) (*AercServer, error) {
 	sockpath := xdg.RuntimePath("aerc.sock")
 	// remove the socket if it is not connected to a session
-	if err := ConnectAndExec(nil); err != nil {
+	if _, err := ConnectAndExec(nil); err != nil {
 		os.Remove(sockpath)
 	}
 	log.Debugf("Starting Unix server: %s", sockpath)
@@ -101,41 +96,9 @@ func (as *AercServer) Serve() {
 }
 
 func (as *AercServer) handleMessage(req *Request) *Response {
-	if len(req.Arguments) == 0 {
-		return &Response{} // send noop success message, i.e. ping
-	}
-	var err error
-	switch {
-	case strings.HasPrefix(req.Arguments[0], "mailto:"):
-		mailto, err := url.Parse(req.Arguments[0])
-		if err != nil {
-			return &Response{Error: err.Error()}
-		}
-		err = as.handler.Mailto(mailto)
-		if err != nil {
-			return &Response{
-				Error: err.Error(),
-			}
-		}
-	case strings.HasPrefix(req.Arguments[0], "mbox:"):
-		err = as.handler.Mbox(req.Arguments[0])
-		if err != nil {
-			return &Response{Error: err.Error()}
-		}
-	case strings.HasPrefix(req.Arguments[0], ":"):
-		if config.General.DisableIPC {
-			return &Response{
-				Error: "command rejected: IPC is disabled",
-			}
-		}
-		cmdline := opt.QuoteArgs(req.Arguments...)
-		err = as.handler.Command(cmdline.String())
-		if err != nil {
-			return &Response{Error: err.Error()}
-		}
-
-	default:
-		return &Response{Error: "command not understood"}
+	err := as.handler.Command(req.Arguments)
+	if err != nil {
+		return &Response{Error: err.Error()}
 	}
 	return &Response{}
 }
