@@ -7,6 +7,7 @@ import (
 	"sync"
 	"time"
 
+	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib/iterator"
 	"git.sr.ht/~rjarry/aerc/lib/marker"
 	"git.sr.ht/~rjarry/aerc/lib/sort"
@@ -18,53 +19,54 @@ import (
 // Accesses to fields must be guarded by MessageStore.Lock/Unlock
 type MessageStore struct {
 	sync.Mutex
-	Deleted  map[uint32]interface{}
-	Messages map[uint32]*models.MessageInfo
+	Name     string
+	Deleted  map[models.UID]interface{}
+	Messages map[models.UID]*models.MessageInfo
 	Sorting  bool
 
+	ui func() *config.UIConfig
+
 	// ctx is given by the directory lister
 	ctx context.Context
 
 	// Ordered list of known UIDs
-	uids    []uint32
+	uids    []models.UID
 	threads []*types.Thread
 
 	// Visible UIDs
 	scrollOffset int
 	scrollLen    int
 
-	selectedUid   uint32
-	bodyCallbacks map[uint32][]func(*types.FullMessage)
+	selectedUid   models.UID
+	bodyCallbacks map[models.UID][]func(*types.FullMessage)
 
 	// marking
 	marker marker.Marker
 
 	// Search/filter results
-	results     []uint32
+	results     []models.UID
 	resultIndex int
 	filter      *types.SearchCriteria
 
 	sortCriteria []*types.SortCriterion
 	sortDefault  []*types.SortCriterion
 
-	threadedView       bool
-	selectLast         bool
-	reverseThreadOrder bool
-	threadContext      bool
-	threadBySubject    bool
-	sortThreadSiblings bool
-	buildThreads       bool
-	builder            *ThreadBuilder
+	threadedView  bool
+	threadContext bool
+	buildThreads  bool
+	builder       *ThreadBuilder
+
+	directoryContentsLoaded bool
 
 	// Map of uids we've asked the worker to fetch
 	onUpdate       func(store *MessageStore) // TODO: multiple onUpdate handlers
 	onFilterChange func(store *MessageStore)
 	onUpdateDirs   func()
-	pendingBodies  map[uint32]interface{}
-	pendingHeaders map[uint32]interface{}
+	pendingBodies  map[models.UID]interface{}
+	pendingHeaders map[models.UID]interface{}
 	worker         *types.Worker
 
-	needsFlags         []uint32
+	needsFlags         []models.UID
 	fetchFlagsDebounce *time.Timer
 	fetchFlagsDelay    time.Duration
 
@@ -76,7 +78,6 @@ type MessageStore struct {
 	triggerFlagChanged     func(string)
 
 	threadBuilderDebounce *time.Timer
-	threadBuilderDelay    time.Duration
 	threadCallback        func()
 
 	// threads mutex protects the store.threads and store.threadCallback
@@ -86,27 +87,22 @@ type MessageStore struct {
 	onSelect    func(*models.MessageInfo)
 }
 
-const MagicUid = 0xFFFFFFFF
+const MagicUid = models.UID("")
 
-func NewMessageStore(worker *types.Worker,
-	defaultSortCriteria []*types.SortCriterion,
-	thread bool, clientThreads bool, clientThreadsDelay time.Duration,
-	selectLast bool, threadBySubject bool,
-	reverseOrder bool, reverseThreadOrder bool, sortThreadSiblings bool,
+func NewMessageStore(worker *types.Worker, name string,
+	ui func() *config.UIConfig,
 	triggerNewEmail func(*models.MessageInfo),
 	triggerDirectoryChange func(), triggerMailDeleted func(),
 	triggerMailAdded func(string), triggerTagModified func([]string, []string),
 	triggerFlagChanged func(string),
 	onSelect func(*models.MessageInfo),
-	threadContext bool,
 ) *MessageStore {
-	if !worker.Backend.Capabilities().Thread {
-		clientThreads = true
-	}
-
 	return &MessageStore{
-		Deleted:  make(map[uint32]interface{}),
-		Messages: make(map[uint32]*models.MessageInfo),
+		Name:     name,
+		Deleted:  make(map[models.UID]interface{}),
+		Messages: make(map[models.UID]*models.MessageInfo),
+
+		ui: ui,
 
 		ctx: context.Background(),
 
@@ -114,24 +110,12 @@ func NewMessageStore(worker *types.Worker,
 		// default window height until account is drawn once
 		scrollLen: 25,
 
-		bodyCallbacks: make(map[uint32][]func(*types.FullMessage)),
-
-		threadedView:       thread,
-		buildThreads:       clientThreads,
-		threadContext:      threadContext,
-		threadBySubject:    threadBySubject,
-		selectLast:         selectLast,
-		reverseThreadOrder: reverseThreadOrder,
-		sortThreadSiblings: sortThreadSiblings,
-
-		sortCriteria: defaultSortCriteria,
-		sortDefault:  defaultSortCriteria,
-
-		pendingBodies:  make(map[uint32]interface{}),
-		pendingHeaders: make(map[uint32]interface{}),
+		bodyCallbacks:  make(map[models.UID][]func(*types.FullMessage)),
+		pendingBodies:  make(map[models.UID]interface{}),
+		pendingHeaders: make(map[models.UID]interface{}),
 		worker:         worker,
 
-		needsFlags:      []uint32{},
+		needsFlags:      []models.UID{},
 		fetchFlagsDelay: 50 * time.Millisecond,
 
 		triggerNewEmail:        triggerNewEmail,
@@ -141,10 +125,28 @@ func NewMessageStore(worker *types.Worker,
 		triggerTagModified:     triggerTagModified,
 		triggerFlagChanged:     triggerFlagChanged,
 
-		threadBuilderDelay: clientThreadsDelay,
+		onSelect: onSelect,
+	}
+}
 
-		iterFactory: iterator.NewFactory(reverseOrder),
-		onSelect:    onSelect,
+func (store *MessageStore) Configure(
+	defaultSort []*types.SortCriterion,
+) {
+	uiConf := store.ui()
+
+	store.buildThreads = uiConf.ForceClientThreads ||
+		!store.worker.Backend.Capabilities().Thread
+	store.iterFactory = iterator.NewFactory(uiConf.ReverseOrder)
+
+	// The following config values can be toggled by the user;
+	// reset to default values when reloading config
+	store.threadedView = uiConf.ThreadingEnabled
+	store.threadContext = uiConf.ThreadContext
+
+	// update the default sort criteria
+	store.sortDefault = defaultSort
+	if store.sortCriteria == nil {
+		store.sortCriteria = defaultSort
 	}
 }
 
@@ -157,12 +159,12 @@ func (store *MessageStore) UpdateScroll(offset, length int) {
 	store.scrollLen = length
 }
 
-func (store *MessageStore) FetchHeaders(uids []uint32,
+func (store *MessageStore) FetchHeaders(uids []models.UID,
 	cb func(types.WorkerMessage),
 ) {
 	// TODO: this could be optimized by pre-allocating toFetch and trimming it
 	// at the end. In practice we expect to get most messages back in one frame.
-	var toFetch []uint32
+	var toFetch []models.UID
 	for _, uid := range uids {
 		if _, ok := store.pendingHeaders[uid]; !ok {
 			toFetch = append(toFetch, uid)
@@ -188,10 +190,10 @@ func (store *MessageStore) FetchHeaders(uids []uint32,
 	}
 }
 
-func (store *MessageStore) FetchFull(uids []uint32, cb func(*types.FullMessage)) {
+func (store *MessageStore) FetchFull(uids []models.UID, cb func(*types.FullMessage)) {
 	// TODO: this could be optimized by pre-allocating toFetch and trimming it
 	// at the end. In practice we expect to get most messages back in one frame.
-	var toFetch []uint32
+	var toFetch []models.UID
 	for _, uid := range uids {
 		if _, ok := store.pendingBodies[uid]; !ok {
 			toFetch = append(toFetch, uid)
@@ -219,7 +221,7 @@ func (store *MessageStore) FetchFull(uids []uint32, cb func(*types.FullMessage))
 	}
 }
 
-func (store *MessageStore) FetchBodyPart(uid uint32, part []int, cb func(io.Reader)) {
+func (store *MessageStore) FetchBodyPart(uid models.UID, part []int, cb func(io.Reader)) {
 	store.worker.PostAction(&types.FetchMessageBodyPart{
 		Uid:  uid,
 		Part: part,
@@ -252,10 +254,11 @@ func merge(to *models.MessageInfo, from *models.MessageInfo) {
 }
 
 func (store *MessageStore) Update(msg types.WorkerMessage) {
-	var newUids []uint32
+	var newUids []models.UID
 	update := false
 	updateThreads := false
 	directoryChange := false
+	directoryContentsWasLoaded := store.directoryContentsLoaded
 	start := store.scrollOffset
 	end := store.scrollOffset + store.scrollLen
 
@@ -264,7 +267,7 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 		store.Sort(store.sortCriteria, nil)
 		update = true
 	case *types.DirectoryContents:
-		newMap := make(map[uint32]*models.MessageInfo, len(msg.Uids))
+		newMap := make(map[models.UID]*models.MessageInfo, len(msg.Uids))
 		for i, uid := range msg.Uids {
 			if msg, ok := store.Messages[uid]; ok {
 				newMap[uid] = msg
@@ -281,15 +284,17 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 		if store.threadedView {
 			store.runThreadBuilderNow()
 		}
+		store.directoryContentsLoaded = true
 	case *types.DirectoryThreaded:
 		if store.builder == nil {
-			store.builder = NewThreadBuilder(store.iterFactory, store.threadBySubject)
+			store.builder = NewThreadBuilder(store.iterFactory,
+				store.ui().ThreadingBySubject)
 		}
-		store.builder.RebuildUids(msg.Threads, store.reverseThreadOrder)
+		store.builder.RebuildUids(msg.Threads, store.ReverseThreadOrder())
 		store.uids = store.builder.Uids()
 		store.threads = msg.Threads
 
-		newMap := make(map[uint32]*models.MessageInfo, len(store.uids))
+		newMap := make(map[models.UID]*models.MessageInfo, len(store.uids))
 		for i, uid := range store.uids {
 			if msg, ok := store.Messages[uid]; ok {
 				newMap[uid] = msg
@@ -304,6 +309,7 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 
 		store.Messages = newMap
 		update = true
+		store.directoryContentsLoaded = true
 	case *types.MessageInfo:
 		infoUpdated := msg.Info.Envelope != nil || msg.Info.Error != nil
 		if existing, ok := store.Messages[msg.Info.Uid]; ok && existing != nil {
@@ -349,13 +355,13 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 			break
 		}
 
-		toDelete := make(map[uint32]interface{})
+		toDelete := make(map[models.UID]interface{})
 		for _, uid := range msg.Uids {
 			toDelete[uid] = nil
 			delete(store.Messages, uid)
 			delete(store.Deleted, uid)
 		}
-		uids := make([]uint32, 0, len(store.uids)-len(msg.Uids))
+		uids := make([]models.UID, 0, len(store.uids)-len(msg.Uids))
 		for _, uid := range store.uids {
 			if _, deleted := toDelete[uid]; deleted {
 				continue
@@ -367,7 +373,7 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 			store.Select(MagicUid)
 		}
 
-		var newResults []uint32
+		var newResults []models.UID
 		for _, res := range store.results {
 			if _, deleted := toDelete[res]; !deleted {
 				newResults = append(newResults, res)
@@ -391,13 +397,13 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 		store.update(updateThreads)
 	}
 
-	if directoryChange && store.triggerDirectoryChange != nil {
+	if directoryContentsWasLoaded && directoryChange && store.triggerDirectoryChange != nil {
 		store.triggerDirectoryChange()
 	}
 
 	if len(newUids) > 0 {
 		store.FetchHeaders(newUids, nil)
-		if store.triggerDirectoryChange != nil {
+		if directoryContentsWasLoaded && store.triggerDirectoryChange != nil {
 			store.triggerDirectoryChange()
 		}
 	}
@@ -428,21 +434,19 @@ func (store *MessageStore) update(threads bool) {
 			store.runThreadBuilder()
 		default:
 			if store.builder == nil {
-				store.builder = NewThreadBuilder(store.iterFactory, store.threadBySubject)
+				store.builder = NewThreadBuilder(store.iterFactory,
+					store.ui().ThreadingBySubject)
 			}
 			store.threadsMutex.Lock()
-			store.builder.RebuildUids(store.threads, store.reverseThreadOrder)
+			store.builder.RebuildUids(store.threads,
+				store.ReverseThreadOrder())
 			store.threadsMutex.Unlock()
 		}
 	}
 }
 
-func (store *MessageStore) SetReverseThreadOrder(reverse bool) {
-	store.reverseThreadOrder = reverse
-}
-
 func (store *MessageStore) ReverseThreadOrder() bool {
-	return store.reverseThreadOrder
+	return store.ui().ReverseThreadOrder
 }
 
 func (store *MessageStore) SetThreadedView(thread bool) {
@@ -482,7 +486,8 @@ func (store *MessageStore) BuildThreads() bool {
 
 func (store *MessageStore) runThreadBuilder() {
 	if store.builder == nil {
-		store.builder = NewThreadBuilder(store.iterFactory, store.threadBySubject)
+		store.builder = NewThreadBuilder(store.iterFactory,
+			store.ui().ThreadingBySubject)
 		for _, msg := range store.Messages {
 			store.builder.Update(msg)
 		}
@@ -490,23 +495,26 @@ func (store *MessageStore) runThreadBuilder() {
 	if store.threadBuilderDebounce != nil {
 		store.threadBuilderDebounce.Stop()
 	}
-	store.threadBuilderDebounce = time.AfterFunc(store.threadBuilderDelay, func() {
-		store.runThreadBuilderNow()
-		ui.Invalidate()
-	})
+	store.threadBuilderDebounce = time.AfterFunc(store.ui().ClientThreadsDelay,
+		func() {
+			store.runThreadBuilderNow()
+			ui.Invalidate()
+		},
+	)
 }
 
 // runThreadBuilderNow runs the threadbuilder without any debounce logic
 func (store *MessageStore) runThreadBuilderNow() {
 	if store.builder == nil {
-		store.builder = NewThreadBuilder(store.iterFactory, store.threadBySubject)
+		store.builder = NewThreadBuilder(store.iterFactory,
+			store.ui().ThreadingBySubject)
 		for _, msg := range store.Messages {
 			store.builder.Update(msg)
 		}
 	}
 	// build new threads
-	th := store.builder.Threads(store.uids, store.reverseThreadOrder,
-		store.sortThreadSiblings)
+	th := store.builder.Threads(store.uids, store.ReverseThreadOrder(),
+		store.ui().SortThreadSiblings)
 
 	// save local threads to the message store variable and
 	// run callback if defined (callback should reposition cursor)
@@ -524,7 +532,7 @@ func (store *MessageStore) runThreadBuilderNow() {
 }
 
 // Thread returns the thread for the given UId
-func (store *MessageStore) Thread(uid uint32) (*types.Thread, error) {
+func (store *MessageStore) Thread(uid models.UID) (*types.Thread, error) {
 	if store.builder == nil {
 		return nil, errors.New("no threads found")
 	}
@@ -536,15 +544,15 @@ func (store *MessageStore) SelectedThread() (*types.Thread, error) {
 	return store.Thread(store.SelectedUid())
 }
 
-func (store *MessageStore) Fold(uid uint32, toggle bool) error {
+func (store *MessageStore) Fold(uid models.UID, toggle bool) error {
 	return store.doThreadFolding(uid, true, toggle)
 }
 
-func (store *MessageStore) Unfold(uid uint32, toggle bool) error {
+func (store *MessageStore) Unfold(uid models.UID, toggle bool) error {
 	return store.doThreadFolding(uid, false, toggle)
 }
 
-func (store *MessageStore) doThreadFolding(uid uint32, hide bool, toggle bool) error {
+func (store *MessageStore) doThreadFolding(uid models.UID, hide bool, toggle bool) error {
 	thread, err := store.Thread(uid)
 	if err != nil {
 		return err
@@ -592,7 +600,7 @@ func (store *MessageStore) doThreadFolding(uid uint32, hide bool, toggle bool) e
 	return nil
 }
 
-func (store *MessageStore) Delete(uids []uint32, mfs *types.MultiFileStrategy,
+func (store *MessageStore) Delete(uids []models.UID, mfs *types.MultiFileStrategy,
 	cb func(msg types.WorkerMessage),
 ) {
 	for _, uid := range uids {
@@ -614,13 +622,13 @@ func (store *MessageStore) Delete(uids []uint32, mfs *types.MultiFileStrategy,
 		})
 }
 
-func (store *MessageStore) revertDeleted(uids []uint32) {
+func (store *MessageStore) revertDeleted(uids []models.UID) {
 	for _, uid := range uids {
 		delete(store.Deleted, uid)
 	}
 }
 
-func (store *MessageStore) Copy(uids []uint32, dest string, createDest bool,
+func (store *MessageStore) Copy(uids []models.UID, dest string, createDest bool,
 	mfs *types.MultiFileStrategy, cb func(msg types.WorkerMessage),
 ) {
 	if createDest {
@@ -642,7 +650,7 @@ func (store *MessageStore) Copy(uids []uint32, dest string, createDest bool,
 	})
 }
 
-func (store *MessageStore) Move(uids []uint32, dest string, createDest bool,
+func (store *MessageStore) Move(uids []models.UID, dest string, createDest bool,
 	mfs *types.MultiFileStrategy, cb func(msg types.WorkerMessage),
 ) {
 	for _, uid := range uids {
@@ -695,7 +703,7 @@ func (store *MessageStore) Append(dest string, flags models.Flags, date time.Tim
 	})
 }
 
-func (store *MessageStore) Flag(uids []uint32, flags models.Flags,
+func (store *MessageStore) Flag(uids []models.UID, flags models.Flags,
 	enable bool, cb func(msg types.WorkerMessage),
 ) {
 	store.worker.PostAction(&types.FlagMessages{
@@ -709,6 +717,8 @@ func (store *MessageStore) Flag(uids []uint32, flags models.Flags,
 			flagName = "seen"
 		case models.AnsweredFlag:
 			flagName = "answered"
+		case models.ForwardedFlag:
+			flagName = "forwarded"
 		case models.FlaggedFlag:
 			flagName = "flagged"
 		case models.DraftFlag:
@@ -723,7 +733,7 @@ func (store *MessageStore) Flag(uids []uint32, flags models.Flags,
 	})
 }
 
-func (store *MessageStore) Answered(uids []uint32, answered bool,
+func (store *MessageStore) Answered(uids []models.UID, answered bool,
 	cb func(msg types.WorkerMessage),
 ) {
 	store.worker.PostAction(&types.AnsweredMessages{
@@ -732,7 +742,16 @@ func (store *MessageStore) Answered(uids []uint32, answered bool,
 	}, cb)
 }
 
-func (store *MessageStore) Uids() []uint32 {
+func (store *MessageStore) Forwarded(uids []models.UID, forwarded bool,
+	cb func(msg types.WorkerMessage),
+) {
+	store.worker.PostAction(&types.ForwardedMessages{
+		Forwarded: forwarded,
+		Uids:      uids,
+	}, cb)
+}
+
+func (store *MessageStore) Uids() []models.UID {
 	if store.ThreadedView() && store.builder != nil {
 		if uids := store.builder.Uids(); len(uids) > 0 {
 			return uids
@@ -749,11 +768,11 @@ func (store *MessageStore) Selected() *models.MessageInfo {
 	return store.Messages[store.selectedUid]
 }
 
-func (store *MessageStore) SelectedUid() uint32 {
+func (store *MessageStore) SelectedUid() models.UID {
 	if store.selectedUid == MagicUid && len(store.Uids()) > 0 {
 		iter := store.UidsIterator()
 		idx := iter.StartIndex()
-		if store.selectLast {
+		if store.ui().SelectLast {
 			idx = iter.EndIndex()
 		}
 		store.Select(store.Uids()[idx])
@@ -761,14 +780,14 @@ func (store *MessageStore) SelectedUid() uint32 {
 	return store.selectedUid
 }
 
-func (store *MessageStore) Select(uid uint32) {
+func (store *MessageStore) Select(uid models.UID) {
 	store.selectPriv(uid, false)
 	if store.onSelect != nil {
 		store.onSelect(store.Selected())
 	}
 }
 
-func (store *MessageStore) selectPriv(uid uint32, lockHeld bool) {
+func (store *MessageStore) selectPriv(uid models.UID, lockHeld bool) {
 	if !lockHeld {
 		store.threadsMutex.Lock()
 	}
@@ -829,14 +848,14 @@ func (store *MessageStore) Prev() {
 	store.NextPrev(-1)
 }
 
-func (store *MessageStore) Search(terms *types.SearchCriteria, cb func([]uint32)) {
+func (store *MessageStore) Search(terms *types.SearchCriteria, cb func([]models.UID)) {
 	store.worker.PostAction(&types.SearchDirectory{
 		Context:  store.ctx,
 		Criteria: terms,
 	}, func(msg types.WorkerMessage) {
 		if msg, ok := msg.(*types.SearchResults); ok {
 			allowedUids := store.Uids()
-			uids := make([]uint32, 0, len(msg.Uids))
+			uids := make([]models.UID, 0, len(msg.Uids))
 			for _, uid := range msg.Uids {
 				for _, uidCheck := range allowedUids {
 					if uid == uidCheck {
@@ -851,14 +870,14 @@ func (store *MessageStore) Search(terms *types.SearchCriteria, cb func([]uint32)
 	})
 }
 
-func (store *MessageStore) ApplySearch(results []uint32) {
+func (store *MessageStore) ApplySearch(results []models.UID) {
 	store.results = results
 	store.resultIndex = -1
 	store.NextResult()
 }
 
 // IsResult returns true if uid is a search result
-func (store *MessageStore) IsResult(uid uint32) bool {
+func (store *MessageStore) IsResult(uid models.UID) bool {
 	for _, hit := range store.results {
 		if hit == uid {
 			return true
@@ -920,7 +939,7 @@ func (store *MessageStore) PrevResult() {
 	store.nextPrevResult(-1)
 }
 
-func (store *MessageStore) ModifyLabels(uids []uint32, add, remove []string,
+func (store *MessageStore) ModifyLabels(uids []models.UID, add, remove []string,
 	cb func(msg types.WorkerMessage),
 ) {
 	store.worker.PostAction(&types.ModifyLabels{
@@ -984,7 +1003,7 @@ func (store *MessageStore) Marker() marker.Marker {
 }
 
 // FindIndexByUid returns the index in store.Uids() or -1 if not found
-func (store *MessageStore) FindIndexByUid(uid uint32) int {
+func (store *MessageStore) FindIndexByUid(uid models.UID) int {
 	for idx, u := range store.Uids() {
 		if u == uid {
 			return idx
@@ -1014,7 +1033,7 @@ func (store *MessageStore) fetchFlags() {
 			Context: store.ctx,
 			Uids:    store.needsFlags,
 		}, nil)
-		store.needsFlags = []uint32{}
+		store.needsFlags = []models.UID{}
 		store.Unlock()
 	})
 }
