@@ -3,6 +3,7 @@ package state
 import (
 	"bufio"
 	"bytes"
+	"errors"
 	"fmt"
 	"os"
 	"os/exec"
@@ -161,6 +162,13 @@ func (d *templateData) AccountBackend() string {
 	return ""
 }
 
+func (d *templateData) AccountFrom() *mail.Address {
+	if d.account != nil {
+		return d.account.From
+	}
+	return nil
+}
+
 func (d *templateData) Folder() string {
 	if d.folder != nil {
 		return d.folder.Name
@@ -410,6 +418,9 @@ func (d *templateData) Flags() []string {
 	if d.info.Flags.Has(models.DeletedFlag) {
 		flags = append(flags, d.ui().IconDeleted)
 	}
+	if d.info.Flags.Has(models.ForwardedFlag) {
+		flags = append(flags, d.ui().IconForwarded)
+	}
 	if d.info.BodyStructure != nil {
 		for _, bS := range d.info.BodyStructure.Parts {
 			if strings.ToLower(bS.Disposition) == "attachment" {
@@ -434,6 +445,13 @@ func (d *templateData) IsReplied() bool {
 	return false
 }
 
+func (d *templateData) IsForwarded() bool {
+	if d.info != nil && d.info.Flags.Has(models.ForwardedFlag) {
+		return true
+	}
+	return false
+}
+
 func (d *templateData) HasAttachment() bool {
 	if d.info != nil && d.info.BodyStructure != nil {
 		for _, bS := range d.info.BodyStructure.Parts {
@@ -691,6 +709,10 @@ func (d *templateData) Signature() string {
 		var err error
 		signature, err = d.readSignatureFromCmd()
 		if err != nil {
+			var execErr *exec.ExitError
+			if errors.As(err, &execErr) {
+				log.Warnf("signature command failed with error (%d): %s", execErr.ExitCode(), execErr.Stderr)
+			}
 			signature = d.readSignatureFromFile()
 		}
 	} else {
@@ -706,6 +728,10 @@ func (d *templateData) Signature() string {
 func (d *templateData) readSignatureFromCmd() ([]byte, error) {
 	sigCmd := d.account.SignatureCmd
 	cmd := exec.Command("sh", "-c", sigCmd)
+	env := os.Environ()
+	env = append(env, fmt.Sprintf("AERC_ACCOUNT=%s", d.account.Name))
+	env = append(env, fmt.Sprintf("AERC_FOLDER=%s", d.folder.Name))
+	cmd.Env = env
 	signature, err := cmd.Output()
 	if err != nil {
 		return nil, err
