@@ -12,6 +12,7 @@ type texterInterface interface {
 	Passthrough() string
 	Sorting() string
 	Threading() string
+	Visual() string
 	FormatFilter(string) string
 	FormatSearch(string) string
 }
@@ -40,6 +41,10 @@ func (t text) Threading() string {
 	return "threading"
 }
 
+func (t text) Visual() string {
+	return "visual"
+}
+
 func (t text) FormatFilter(s string) string {
 	return s
 }
@@ -72,6 +77,10 @@ func (i icon) Threading() string {
 	return "🧵"
 }
 
+func (i icon) Visual() string {
+	return "🕶"
+}
+
 func (i icon) FormatFilter(s string) string {
 	return strings.ReplaceAll(s, "filter", "🔦")
 }
