@@ -6,6 +6,7 @@ import (
 	"fmt"
 	"os"
 	"os/exec"
+	"path/filepath"
 	"regexp"
 	"strings"
 	"text/template"
@@ -24,6 +25,18 @@ func SetVersion(v string) {
 	version = v
 }
 
+var execPath string
+
+func SetExecPath(dirs []string) {
+	// prepend aerc filters dirs to the default exec path
+	paths := make([]string, 0, len(dirs)+1)
+	for _, d := range dirs {
+		paths = append(paths, filepath.Join(d, "filters"))
+	}
+	paths = append(paths, os.Getenv("PATH"))
+	execPath = strings.Join(paths, ":")
+}
+
 // wrap allows to chain wrapText
 func wrap(lineWidth int, text string) string {
 	return wrapText(text, lineWidth)
@@ -84,7 +97,11 @@ func quote(text string) string {
 			quoted.WriteString(">\n")
 			continue
 		}
-		quoted.WriteString("> ")
+		if strings.HasPrefix(line, ">") {
+			quoted.WriteString(">")
+		} else {
+			quoted.WriteString("> ")
+		}
 		quoted.WriteString(line)
 		quoted.WriteRune('\n')
 	}
@@ -98,6 +115,7 @@ func quote(text string) string {
 func cmd(cmd, text string) string {
 	var out bytes.Buffer
 	c := exec.Command("sh", "-c", cmd)
+	c.Env = append(os.Environ(), "PATH="+execPath)
 	c.Stdin = strings.NewReader(text)
 	c.Stdout = &out
 	err := c.Run()
@@ -334,6 +352,24 @@ func hasPrefix(prefix, s string) bool {
 	return strings.HasPrefix(s, prefix)
 }
 
+func head(n uint, s string) string {
+	r := []rune(s)
+	length := uint(len(r))
+	if length >= n {
+		return string(r[:n])
+	}
+	return s
+}
+
+func tail(n uint, s string) string {
+	r := []rune(s)
+	length := uint(len(r))
+	if length >= n {
+		return string(r[length-n:])
+	}
+	return s
+}
+
 var templateFuncs = template.FuncMap{
 	"quote":         quote,
 	"wrapText":      wrapText,
@@ -367,4 +403,6 @@ var templateFuncs = template.FuncMap{
 	"toLower":       strings.ToLower,
 	"toUpper":       strings.ToUpper,
 	"replace":       replace,
+	"head":          head,
+	"tail":          tail,
 }
