@@ -1,16 +1,18 @@
 package ui
 
 import (
+	"context"
 	"math"
 	"strings"
 	"sync"
 	"time"
 
-	"github.com/gdamore/tcell/v2"
 	"github.com/mattn/go-runewidth"
 
 	"git.sr.ht/~rjarry/aerc/config"
-	"git.sr.ht/~rjarry/aerc/log"
+	"git.sr.ht/~rjarry/aerc/lib/log"
+	"git.sr.ht/~rjarry/go-opt/v2"
+	"git.sr.ht/~rockorager/vaxis"
 )
 
 // TODO: Attach history providers
@@ -24,11 +26,12 @@ type TextInput struct {
 	password          bool
 	prompt            string
 	scroll            int
-	text              []rune
+	text              []vaxis.Character
 	change            []func(ti *TextInput)
 	focusLost         []func(ti *TextInput)
-	tabcomplete       func(s string) ([]string, string)
-	completions       []string
+	tabcomplete       func(ctx context.Context, s string) ([]opt.Completion, string)
+	tabcompleteCancel context.CancelFunc
+	completions       []opt.Completion
 	prefix            string
 	completeIndex     int
 	completeDelay     time.Duration
@@ -42,11 +45,13 @@ type TextInput struct {
 // context they're given, and process keypresses to build a string from user
 // input.
 func NewTextInput(text string, ui *config.UIConfig) *TextInput {
+	chars := vaxis.Characters(text)
 	return &TextInput{
-		cells:    -1,
-		text:     []rune(text),
-		index:    len([]rune(text)),
-		uiConfig: ui,
+		cells:             -1,
+		text:              chars,
+		index:             len(chars),
+		uiConfig:          ui,
+		tabcompleteCancel: func() {},
 	}
 }
 
@@ -61,7 +66,7 @@ func (ti *TextInput) Prompt(prompt string) *TextInput {
 }
 
 func (ti *TextInput) TabComplete(
-	tabcomplete func(s string) ([]string, string),
+	tabcomplete func(ctx context.Context, s string) ([]opt.Completion, string),
 	d time.Duration, minChars int, key *config.KeyStroke,
 ) *TextInput {
 	ti.tabcomplete = tabcomplete
@@ -72,22 +77,35 @@ func (ti *TextInput) TabComplete(
 }
 
 func (ti *TextInput) String() string {
-	return string(ti.text)
+	return charactersToString(ti.text)
 }
 
 func (ti *TextInput) StringLeft() string {
-	for ti.index > len(ti.text) {
+	if ti.index > len(ti.text) {
 		ti.index = len(ti.text)
 	}
-	return string(ti.text[:ti.index])
+	left := ti.text[:ti.index]
+	return charactersToString(left)
 }
 
 func (ti *TextInput) StringRight() string {
-	return string(ti.text[ti.index:])
+	if ti.index >= len(ti.text) {
+		return ""
+	}
+	right := ti.text[ti.index:]
+	return charactersToString(right)
+}
+
+func charactersToString(chars []vaxis.Character) string {
+	buf := strings.Builder{}
+	for _, ch := range chars {
+		buf.WriteString(ch.Grapheme)
+	}
+	return buf.String()
 }
 
 func (ti *TextInput) Set(value string) *TextInput {
-	ti.text = []rune(value)
+	ti.text = vaxis.Characters(value)
 	ti.index = len(ti.text)
 	ti.scroll = 0
 	return ti
@@ -112,14 +130,14 @@ func (ti *TextInput) Draw(ctx *Context) {
 	sindex := ti.index - scroll
 	if ti.password {
 		x := ctx.Printf(0, 0, defaultStyle, "%s", ti.prompt)
-		cells := runewidth.StringWidth(string(text))
+		cells := len(ti.text)
 		ctx.Fill(x, 0, cells, 1, '*', defaultStyle)
 	} else {
-		ctx.Printf(0, 0, defaultStyle, "%s%s", ti.prompt, string(text))
+		ctx.Printf(0, 0, defaultStyle, "%s%s", ti.prompt, charactersToString(text))
 	}
-	cells := runewidth.StringWidth(string(text[:sindex]) + ti.prompt)
+	cells := runewidth.StringWidth(charactersToString(text[:sindex]) + ti.prompt)
 	if ti.focus {
-		ctx.SetCursor(cells, 0)
+		ctx.SetCursor(cells, 0, vaxis.CursorDefault)
 		ti.drawPopover(ctx)
 	}
 }
@@ -128,8 +146,24 @@ func (ti *TextInput) drawPopover(ctx *Context) {
 	if len(ti.completions) == 0 {
 		return
 	}
-	cmp := &completions{ti: ti}
-	width := maxLen(ti.completions) + 3
+
+	valWidth := 0
+	descWidth := 0
+	for _, c := range ti.completions {
+		valWidth = max(valWidth, runewidth.StringWidth(unquote(c.Value)))
+		descWidth = max(descWidth, runewidth.StringWidth(c.Description))
+	}
+	descWidth = min(descWidth, 80)
+	// one space padding
+	width := 1 + valWidth
+	if descWidth != 0 {
+		// two spaces padding + parentheses
+		width += 2 + descWidth + 2
+	}
+	// one space padding + gutter
+	width += 2
+
+	cmp := &completions{ti: ti, valWidth: valWidth, descWidth: descWidth}
 	height := len(ti.completions)
 
 	pos := len(ti.prefix) - ti.scroll
@@ -143,9 +177,9 @@ func (ti *TextInput) drawPopover(ctx *Context) {
 	ctx.Popover(pos, 0, width, height, cmp)
 }
 
-func (ti *TextInput) MouseEvent(localX int, localY int, event tcell.Event) {
-	if event, ok := event.(*tcell.EventMouse); ok {
-		if event.Buttons() == tcell.Button1 {
+func (ti *TextInput) MouseEvent(localX int, localY int, event vaxis.Event) {
+	if event, ok := event.(vaxis.Mouse); ok {
+		if event.Button == vaxis.MouseLeftButton {
 			if localX >= len(ti.prompt)+1 && localX <= len(ti.text[ti.scroll:])+len(ti.prompt)+1 {
 				ti.index = localX - len(ti.prompt) - 1
 				ti.ensureScroll()
@@ -161,8 +195,8 @@ func (ti *TextInput) Focus(focus bool) {
 	}
 	ti.focus = focus
 	if focus && ti.ctx != nil {
-		cells := runewidth.StringWidth(string(ti.text[:ti.index]))
-		ti.ctx.SetCursor(cells+1, 0)
+		cells := runewidth.StringWidth(charactersToString(ti.text[:ti.index]))
+		ti.ctx.SetCursor(cells+1, 0, vaxis.CursorDefault)
 	} else if !focus && ti.ctx != nil {
 		ti.ctx.HideCursor()
 	}
@@ -181,10 +215,10 @@ func (ti *TextInput) ensureScroll() {
 	}
 }
 
-func (ti *TextInput) insert(ch rune) {
+func (ti *TextInput) insert(ch vaxis.Character) {
 	left := ti.text[:ti.index]
 	right := ti.text[ti.index:]
-	ti.text = append(left, append([]rune{ch}, right...)...) //nolint:gocritic // intentional append to different slice
+	ti.text = append(left, append([]vaxis.Character{ch}, right...)...) //nolint:gocritic // intentional append to different slice
 	ti.index++
 	ti.ensureScroll()
 	ti.Invalidate()
@@ -197,16 +231,16 @@ func (ti *TextInput) deleteWord() {
 	}
 	separators := "/'\""
 	i := ti.index - 1
-	for i >= 0 && ti.text[i] == ' ' {
+	for i >= 0 && ti.text[i].Grapheme == " " {
 		i--
 	}
-	if i >= 0 && strings.ContainsRune(separators, ti.text[i]) {
-		for i >= 0 && strings.ContainsRune(separators, ti.text[i]) {
+	if i >= 0 && strings.Contains(separators, ti.text[i].Grapheme) {
+		for i >= 0 && strings.Contains(separators, ti.text[i].Grapheme) {
 			i--
 		}
 	} else {
 		separators += " "
-		for i >= 0 && !strings.ContainsRune(separators, ti.text[i]) {
+		for i >= 0 && !strings.Contains(separators, ti.text[i].Grapheme) {
 			i--
 		}
 	}
@@ -261,7 +295,7 @@ func (ti *TextInput) backspace() {
 
 func (ti *TextInput) executeCompletion() {
 	if len(ti.completions) > 0 {
-		ti.Set(ti.prefix + ti.completions[ti.completeIndex] + ti.StringRight())
+		ti.Set(ti.prefix + ti.completions[ti.completeIndex].Value + ti.StringRight())
 	}
 }
 
@@ -311,17 +345,34 @@ func (ti *TextInput) showCompletions(explicit bool) {
 		// no completer
 		return
 	}
-	ti.completions, ti.prefix = ti.tabcomplete(ti.StringLeft())
-
-	if explicit && len(ti.completions) == 1 {
-		// automatically accept if there is only one choice
-		ti.completeIndex = 0
-		ti.executeCompletion()
-		ti.invalidateCompletions()
-	} else {
-		ti.completeIndex = -1
+	if ti.tabcompleteCancel != nil {
+		// Cancel any inflight completions we currently have
+		ti.tabcompleteCancel()
 	}
-	Invalidate()
+	ctx, cancel := context.WithCancel(context.Background())
+	ti.tabcompleteCancel = cancel
+	go func() {
+		defer log.PanicHandler()
+		matches, prefix := ti.tabcomplete(ctx, ti.StringLeft())
+		select {
+		case <-ctx.Done():
+			return
+		default:
+			ti.Lock()
+			defer ti.Unlock()
+			ti.completions = matches
+			ti.prefix = prefix
+			if explicit && len(ti.completions) == 1 {
+				// automatically accept if there is only one choice
+				ti.completeIndex = 0
+				ti.executeCompletion()
+				ti.invalidateCompletions()
+			} else {
+				ti.completeIndex = -1
+			}
+			Invalidate()
+		}
+	}()
 }
 
 func (ti *TextInput) OnChange(onChange func(ti *TextInput)) {
@@ -332,60 +383,65 @@ func (ti *TextInput) OnFocusLost(onFocusLost func(ti *TextInput)) {
 	ti.focusLost = append(ti.focusLost, onFocusLost)
 }
 
-func (ti *TextInput) Event(event tcell.Event) bool {
+func (ti *TextInput) Event(event vaxis.Event) bool {
 	ti.Lock()
 	defer ti.Unlock()
-	if event, ok := event.(*tcell.EventKey); ok {
+	if key, ok := event.(vaxis.Key); ok {
 		c := ti.completeKey
-		if c != nil && c.Key == event.Key() && c.Modifiers == event.Modifiers() {
+		if c != nil && key.Matches(c.Key, c.Modifiers) {
 			ti.showCompletions(true)
 			return true
 		}
 
 		ti.invalidateCompletions()
 
-		switch event.Key() {
-		case tcell.KeyBackspace, tcell.KeyBackspace2:
+		switch {
+		case key.Matches(vaxis.KeyBackspace):
 			ti.backspace()
-		case tcell.KeyCtrlD, tcell.KeyDelete:
+		case key.Matches('d', vaxis.ModCtrl), key.Matches(vaxis.KeyDelete):
 			ti.deleteChar()
-		case tcell.KeyCtrlB, tcell.KeyLeft:
+		case key.Matches('b', vaxis.ModCtrl), key.Matches(vaxis.KeyLeft):
 			if ti.index > 0 {
 				ti.index--
 				ti.ensureScroll()
 				ti.Invalidate()
 			}
-		case tcell.KeyCtrlF, tcell.KeyRight:
+		case key.Matches('f', vaxis.ModCtrl), key.Matches(vaxis.KeyRight):
 			if ti.index < len(ti.text) {
 				ti.index++
 				ti.ensureScroll()
 				ti.Invalidate()
 			}
-		case tcell.KeyCtrlA, tcell.KeyHome:
+		case key.Matches('a', vaxis.ModCtrl), key.Matches(vaxis.KeyHome):
 			ti.index = 0
 			ti.ensureScroll()
 			ti.Invalidate()
-		case tcell.KeyCtrlE, tcell.KeyEnd:
+		case key.Matches('e', vaxis.ModCtrl), key.Matches(vaxis.KeyEnd):
 			ti.index = len(ti.text)
 			ti.ensureScroll()
 			ti.Invalidate()
-		case tcell.KeyCtrlK:
+		case key.Matches('k', vaxis.ModCtrl):
 			ti.deleteLineForward()
-		case tcell.KeyCtrlW:
+		case key.Matches('w', vaxis.ModCtrl):
 			ti.deleteWord()
-		case tcell.KeyCtrlU:
+		case key.Matches('u', vaxis.ModCtrl):
 			ti.deleteLineBackward()
-		case tcell.KeyESC:
+		case key.Matches(vaxis.KeyEsc):
 			ti.Invalidate()
-		case tcell.KeyRune:
-			ti.insert(event.Rune())
+		case key.Text != "":
+			chars := vaxis.Characters(key.Text)
+			for _, ch := range chars {
+				ti.insert(ch)
+			}
 		}
 	}
 	return true
 }
 
 type completions struct {
-	ti *TextInput
+	ti        *TextInput
+	valWidth  int
+	descWidth int
 }
 
 func unquote(s string) string {
@@ -395,22 +451,13 @@ func unquote(s string) string {
 	return s
 }
 
-func maxLen(ss []string) int {
-	max := 0
-	for _, s := range ss {
-		l := runewidth.StringWidth(unquote(s))
-		if l > max {
-			max = l
-		}
-	}
-	return max
-}
-
 func (c *completions) Draw(ctx *Context) {
 	bg := c.ti.uiConfig.GetStyle(config.STYLE_COMPLETION_DEFAULT)
+	bgDesc := c.ti.uiConfig.GetStyle(config.STYLE_COMPLETION_DESCRIPTION)
 	gutter := c.ti.uiConfig.GetStyle(config.STYLE_COMPLETION_GUTTER)
 	pill := c.ti.uiConfig.GetStyle(config.STYLE_COMPLETION_PILL)
 	sel := c.ti.uiConfig.GetStyleSelected(config.STYLE_COMPLETION_DEFAULT)
+	selDesc := c.ti.uiConfig.GetStyleSelected(config.STYLE_COMPLETION_DESCRIPTION)
 
 	ctx.Fill(0, 0, ctx.Width(), ctx.Height(), ' ', bg)
 
@@ -428,11 +475,20 @@ func (c *completions) Draw(ctx *Context) {
 		if idx > endIdx {
 			continue
 		}
+		val := runewidth.FillRight(unquote(opt.Value), c.valWidth)
+		desc := opt.Description
+		if desc != "" {
+			if runewidth.StringWidth(desc) > c.descWidth {
+				desc = runewidth.Truncate(desc, c.descWidth, "…")
+			}
+			desc = "  " + runewidth.FillRight("("+desc+")", c.descWidth+2)
+		}
 		if c.index() == idx {
-			ctx.Fill(0, idx-startIdx, ctx.Width(), 1, ' ', sel)
-			ctx.Printf(0, idx-startIdx, sel, " %s ", unquote(opt))
+			n := ctx.Printf(0, idx-startIdx, sel, " %s", val)
+			ctx.Printf(n, idx-startIdx, selDesc, "%s ", desc)
 		} else {
-			ctx.Printf(0, idx-startIdx, bg, " %s ", unquote(opt))
+			n := ctx.Printf(0, idx-startIdx, bg, " %s", val)
+			ctx.Printf(n, idx-startIdx, bgDesc, "%s ", desc)
 		}
 	}
 
@@ -480,10 +536,10 @@ func (c *completions) exec() {
 	Invalidate()
 }
 
-func (c *completions) Event(e tcell.Event) bool {
-	if e, ok := e.(*tcell.EventKey); ok {
+func (c *completions) Event(e vaxis.Event) bool {
+	if e, ok := e.(vaxis.Key); ok {
 		k := c.ti.completeKey
-		if k != nil && k.Key == e.Key() && k.Modifiers == e.Modifiers() {
+		if k != nil && e.Matches(k.Key, k.Modifiers) {
 			if len(c.ti.completions) == 1 {
 				c.ti.completeIndex = 0
 				c.exec()
@@ -497,14 +553,16 @@ func (c *completions) Event(e tcell.Event) bool {
 			return true
 		}
 
-		switch e.Key() {
-		case tcell.KeyCtrlN, tcell.KeyDown:
+		switch {
+		case e.Matches('n', vaxis.ModCtrl), e.Matches(vaxis.KeyDown):
 			c.next()
 			return true
-		case tcell.KeyBacktab, tcell.KeyCtrlP, tcell.KeyUp:
+		case e.Matches(vaxis.KeyTab, vaxis.ModShift),
+			e.Matches('p', vaxis.ModCtrl),
+			e.Matches(vaxis.KeyUp):
 			c.prev()
 			return true
-		case tcell.KeyEnter:
+		case e.Matches(vaxis.KeyEnter):
 			if c.index() >= 0 {
 				c.exec()
 				return true
@@ -526,26 +584,26 @@ func (c *completions) needsStem(stem string) bool {
 
 func (c *completions) stem(stem string) {
 	c.ti.Set(c.ti.prefix + stem + c.ti.StringRight())
-	c.ti.index = runewidth.StringWidth(c.ti.prefix + stem)
+	c.ti.index = len(vaxis.Characters(c.ti.prefix + stem))
 }
 
-func findStem(words []string) string {
+func findStem(words []opt.Completion) string {
 	if len(words) == 0 {
 		return ""
 	}
 	if len(words) == 1 {
-		return words[0]
+		return words[0].Value
 	}
 	var stem string
 	stemLen := 1
-	firstWord := []rune(words[0])
+	firstWord := []rune(words[0].Value)
 	for {
 		if len(firstWord) < stemLen {
 			return stem
 		}
 		var r rune = firstWord[stemLen-1]
 		for _, word := range words[1:] {
-			runes := []rune(word)
+			runes := []rune(word.Value)
 			if len(runes) < stemLen {
 				return stem
 			}
