@@ -4,6 +4,7 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"strconv"
 	"strings"
 	"time"
 
@@ -25,6 +26,9 @@ const (
 	// AnsweredFlag marks a message as having been replied to
 	AnsweredFlag
 
+	// ForwardedFlag marks a message as having been forwarded
+	ForwardedFlag
+
 	// DeletedFlag marks a message as having been deleted
 	DeletedFlag
 
@@ -90,8 +94,45 @@ type DirectoryInfo struct {
 
 // Capabilities provides the backend capabilities
 type Capabilities struct {
-	Sort   bool
-	Thread bool
+	Sort       bool
+	Thread     bool
+	Extensions []string
+}
+
+func (c *Capabilities) Has(s string) bool {
+	for _, ext := range c.Extensions {
+		if ext == s {
+			return true
+		}
+	}
+	return false
+}
+
+type UID string
+
+func UidToUint32(uid UID) uint32 {
+	u, _ := strconv.ParseUint(string(uid), 10, 32)
+	return uint32(u)
+}
+
+func Uint32ToUid(u uint32) UID {
+	return UID(fmt.Sprintf("%012d", u))
+}
+
+func UidToUint32List(uids []UID) []uint32 {
+	ulist := make([]uint32, 0, len(uids))
+	for _, uid := range uids {
+		ulist = append(ulist, UidToUint32(uid))
+	}
+	return ulist
+}
+
+func Uint32ToUidList(ulist []uint32) []UID {
+	uids := make([]UID, 0, len(ulist))
+	for _, u := range ulist {
+		uids = append(uids, Uint32ToUid(u))
+	}
+	return uids
 }
 
 // A MessageInfo holds information about the structure of a message
@@ -105,7 +146,7 @@ type MessageInfo struct {
 	RFC822Headers *mail.Header
 	Refs          []string
 	Size          uint32
-	Uid           uint32
+	Uid           UID
 	Error         error
 }
 
@@ -156,13 +197,13 @@ func (mi *MessageInfo) References() ([]string, error) {
 // A MessageBodyPart can be displayed in the message viewer
 type MessageBodyPart struct {
 	Reader io.Reader
-	Uid    uint32
+	Uid    UID
 }
 
 // A FullMessage is the entire message
 type FullMessage struct {
 	Reader io.Reader
-	Uid    uint32
+	Uid    UID
 }
 
 type BodyStructure struct {
@@ -229,6 +270,7 @@ type Envelope struct {
 	Subject   string
 	From      []*mail.Address
 	ReplyTo   []*mail.Address
+	Sender    []*mail.Address
 	To        []*mail.Address
 	Cc        []*mail.Address
 	Bcc       []*mail.Address
@@ -243,6 +285,7 @@ type OriginalMail struct {
 	Text          string
 	MIMEType      string
 	RFC822Headers *mail.Header
+	Folder        string
 }
 
 type SignatureValidity int32
