@@ -9,6 +9,8 @@ import (
 // This interface needs to be implemented for compliance with aerc-templates(7)
 type TemplateData interface {
 	Account() string
+	AccountBackend() string
+	AccountFrom() *mail.Address
 	Folder() string
 	To() []*mail.Address
 	Cc() []*mail.Address
@@ -24,6 +26,7 @@ type TemplateData interface {
 	ThreadUnread() int
 	ThreadFolded() bool
 	ThreadContext() bool
+	ThreadOrphan() bool
 	Subject() string
 	SubjectBase() string
 	Number() int
@@ -39,6 +42,7 @@ type TemplateData interface {
 	IsUnread() bool
 	IsMarked() bool
 	IsDraft() bool
+	IsForwarded() bool
 	MessageId() string
 	Role() string
 	Size() int
