@@ -1,17 +1,23 @@
 package imap
 
 import (
+	"bufio"
 	"fmt"
 	"net/url"
+	"os"
 	"strconv"
 	"strings"
 	"time"
 
+	"git.sr.ht/~rjarry/aerc/lib/xdg"
+	"git.sr.ht/~rjarry/aerc/worker/lib"
+	"git.sr.ht/~rjarry/aerc/worker/middleware"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 	"golang.org/x/oauth2"
 )
 
 func (w *IMAPWorker) handleConfigure(msg *types.Configure) error {
+	w.config.name = msg.Config.Name
 	u, err := url.Parse(msg.Config.Source)
 	if err != nil {
 		return err
@@ -60,6 +66,8 @@ func (w *IMAPWorker) handleConfigure(msg *types.Configure) error {
 
 	w.config.user = u.User
 	w.config.folders = msg.Config.Folders
+	w.config.headers = msg.Config.Headers
+	w.config.headersExclude = msg.Config.HeadersExclude
 
 	w.config.idle_timeout = 10 * time.Second
 	w.config.idle_debounce = 10 * time.Millisecond
@@ -147,13 +155,33 @@ func (w *IMAPWorker) handleConfigure(msg *types.Configure) error {
 				return fmt.Errorf("invalid cache-max-age value %v: %w", value, err)
 			}
 			w.config.cacheMaxAge = val
+		case "use-gmail-ext":
+			val, err := strconv.ParseBool(value)
+			if err != nil {
+				return fmt.Errorf("invalid use-gmail-ext value %v: %w", value, err)
+			}
+			w.config.useXGMEXT = val
 		}
 	}
 	if w.config.cacheEnabled {
 		w.initCacheDb(msg.Config.Name)
 	}
-	w.idler = newIdler(w.config, w.worker)
+	w.idler = newIdler(w.config, w.worker, w.executeIdle)
 	w.observer = newObserver(w.config, w.worker)
 
+	if name, ok := msg.Config.Params["folder-map"]; ok {
+		file := xdg.ExpandHome(name)
+		f, err := os.Open(file)
+		if err != nil {
+			return err
+		}
+		defer f.Close()
+		fmap, order, err := lib.ParseFolderMap(bufio.NewReader(f))
+		if err != nil {
+			return err
+		}
+		w.worker = middleware.NewFolderMapper(w.worker, fmap, order)
+	}
+
 	return nil
 }
