@@ -128,10 +128,12 @@ func (w *IMAPWorker) newClient(c *client.Client) {
 	lStatus, err := w.client.liststatus.SupportListStatus()
 	if err == nil && lStatus {
 		w.liststatus = true
+		w.caps.Extensions = append(w.caps.Extensions, "LIST-STATUS")
 		w.worker.Debugf("Server Capability found: LIST-STATUS")
 	}
 	xgmext, err := w.client.Support("X-GM-EXT-1")
 	if err == nil && xgmext && w.config.useXGMEXT {
+		w.caps.Extensions = append(w.caps.Extensions, "X-GM-EXT-1")
 		w.worker.Debugf("Server Capability found: X-GM-EXT-1")
 		w.worker = middleware.NewGmailWorker(w.worker, w.client.Client)
 	}
@@ -287,7 +289,7 @@ func (w *IMAPWorker) handleImapUpdate(update client.Update) {
 				Envelope:      translateEnvelope(msg.Envelope),
 				Flags:         translateImapFlags(msg.Flags),
 				InternalDate:  msg.InternalDate,
-				Uid:           msg.Uid,
+				Uid:           models.Uint32ToUid(msg.Uid),
 			},
 		}, nil)
 	case *client.ExpungeUpdate:
@@ -295,7 +297,7 @@ func (w *IMAPWorker) handleImapUpdate(update client.Update) {
 			w.worker.Errorf("ExpungeUpdate unknown seqnum: %d", update.SeqNum)
 		} else {
 			w.worker.PostMessage(&types.MessagesDeleted{
-				Uids: []uint32{uid},
+				Uids: []models.UID{models.Uint32ToUid(uid)},
 			}, nil)
 		}
 	}
