@@ -16,6 +16,7 @@ import (
 var headersProperties = []string{
 	"id",
 	"blobId",
+	"threadId",
 	"mailboxIds",
 	"keywords",
 	"size",
@@ -34,58 +35,88 @@ var headersProperties = []string{
 }
 
 func (w *JMAPWorker) handleFetchMessageHeaders(msg *types.FetchMessageHeaders) error {
-	var req jmap.Request
-
-	ids := make([]jmap.ID, 0, len(msg.Uids))
+	emailIdsToFetch := make([]jmap.ID, 0, len(msg.Uids))
+	currentEmails := make([]*email.Email, 0, len(msg.Uids))
 	for _, uid := range msg.Uids {
-		id, ok := w.uidStore.GetKey(uid)
-		if !ok {
-			return fmt.Errorf("bug: no jmap id for message uid: %v", uid)
-		}
-		jid := jmap.ID(id)
+		jid := jmap.ID(uid)
 		m, err := w.cache.GetEmail(jid)
-		if err == nil {
+		if err != nil {
+			// Message wasn't in cache; fetch it
+			emailIdsToFetch = append(emailIdsToFetch, jid)
+			continue
+		}
+		currentEmails = append(currentEmails, m)
+		// Get the UI updated immediately
+		w.w.PostMessage(&types.MessageInfo{
+			Message: types.RespondTo(msg),
+			Info:    w.translateMsgInfo(m),
+		}, nil)
+	}
+
+	if len(emailIdsToFetch) > 0 {
+		var req jmap.Request
+
+		req.Invoke(&email.Get{
+			Account:    w.AccountId(),
+			IDs:        emailIdsToFetch,
+			Properties: headersProperties,
+		})
+
+		resp, err := w.Do(&req)
+		if err != nil {
+			return err
+		}
+
+		for _, inv := range resp.Responses {
+			switch r := inv.Args.(type) {
+			case *email.GetResponse:
+				if err = w.cache.PutEmailState(r.State); err != nil {
+					w.w.Warnf("PutEmailState: %s", err)
+				}
+				currentEmails = append(currentEmails, r.List...)
+			case *jmap.MethodError:
+				return wrapMethodError(r)
+			}
+		}
+	}
+
+	var threadsToFetch []jmap.ID
+	for _, eml := range currentEmails {
+		thread, err := w.cache.GetThread(eml.ThreadID)
+		if err != nil {
+			threadsToFetch = append(threadsToFetch, eml.ThreadID)
+			continue
+		}
+		for _, id := range thread {
+			m, err := w.cache.GetEmail(id)
+			if err != nil {
+				// This should never happen. If we have the
+				// thread in cache, we will have fetched it
+				// already or updated it from the update loop
+				w.w.Warnf("Email ID %s from Thread %s not in cache", id, eml.ThreadID)
+				continue
+			}
+			currentEmails = append(currentEmails, m)
+			// Get the UI updated immediately
 			w.w.PostMessage(&types.MessageInfo{
 				Message: types.RespondTo(msg),
 				Info:    w.translateMsgInfo(m),
 			}, nil)
-			continue
 		}
-		ids = append(ids, jid)
 	}
 
-	if len(ids) == 0 {
-		return nil
-	}
-
-	req.Invoke(&email.Get{
-		Account:    w.AccountId(),
-		IDs:        ids,
-		Properties: headersProperties,
-	})
-
-	resp, err := w.Do(&req)
+	threadEmails, err := w.fetchEntireThreads(threadsToFetch)
 	if err != nil {
 		return err
 	}
 
-	for _, inv := range resp.Responses {
-		switch r := inv.Args.(type) {
-		case *email.GetResponse:
-			for _, m := range r.List {
-				w.w.PostMessage(&types.MessageInfo{
-					Message: types.RespondTo(msg),
-					Info:    w.translateMsgInfo(m),
-				}, nil)
-				if err := w.cache.PutEmail(m.ID, m); err != nil {
-					w.w.Warnf("PutEmail: %s", err)
-				}
-			}
-			if err = w.cache.PutEmailState(r.State); err != nil {
-				w.w.Warnf("PutEmailState: %s", err)
-			}
-		case *jmap.MethodError:
-			return wrapMethodError(r)
+	for _, m := range threadEmails {
+		w.w.PostMessage(&types.MessageInfo{
+			Message: types.RespondTo(msg),
+			Info:    w.translateMsgInfo(m),
+		}, nil)
+		if err := w.cache.PutEmail(m.ID, m); err != nil {
+			w.w.Warnf("PutEmail: %s", err)
 		}
 	}
 
@@ -93,13 +124,9 @@ func (w *JMAPWorker) handleFetchMessageHeaders(msg *types.FetchMessageHeaders) e
 }
 
 func (w *JMAPWorker) handleFetchMessageBodyPart(msg *types.FetchMessageBodyPart) error {
-	id, ok := w.uidStore.GetKey(msg.Uid)
-	if !ok {
-		return fmt.Errorf("bug: unknown message uid %d", msg.Uid)
-	}
-	mail, err := w.cache.GetEmail(jmap.ID(id))
+	mail, err := w.cache.GetEmail(jmap.ID(msg.Uid))
 	if err != nil {
-		return fmt.Errorf("bug: unknown message id %s: %w", id, err)
+		return fmt.Errorf("bug: unknown message id %s: %w", msg.Uid, err)
 	}
 
 	part := mail.BodyStructure
@@ -132,9 +159,10 @@ func (w *JMAPWorker) handleFetchMessageBodyPart(msg *types.FetchMessageBodyPart)
 	if strings.HasPrefix(part.Type, "text/") && part.Charset != "" {
 		r, err := charset.Reader(part.Charset, reader)
 		if err != nil {
-			return fmt.Errorf("charset.Reader: %w", err)
+			w.w.Warnf("charset.Reader: %v", err)
+		} else {
+			reader = r
 		}
-		reader = r
 	}
 	w.w.PostMessage(&types.MessageBodyPart{
 		Message: types.RespondTo(msg),
@@ -149,13 +177,9 @@ func (w *JMAPWorker) handleFetchMessageBodyPart(msg *types.FetchMessageBodyPart)
 
 func (w *JMAPWorker) handleFetchFullMessages(msg *types.FetchFullMessages) error {
 	for _, uid := range msg.Uids {
-		id, ok := w.uidStore.GetKey(uid)
-		if !ok {
-			return fmt.Errorf("bug: unknown message uid %d", uid)
-		}
-		mail, err := w.cache.GetEmail(jmap.ID(id))
+		mail, err := w.cache.GetEmail(jmap.ID(uid))
 		if err != nil {
-			return fmt.Errorf("bug: unknown message id %s: %w", id, err)
+			return fmt.Errorf("bug: unknown message id %s: %w", uid, err)
 		}
 		buf, err := w.cache.GetBlob(mail.BlobID)
 		if err != nil {
