@@ -66,6 +66,17 @@ func (w *JMAPWorker) handleStartSend(msg *types.StartSendingMessage) error {
 			})
 		}
 		envelope := &emailsubmission.Envelope{MailFrom: from, RcptTo: rcpts}
+		onSuccess := jmap.Patch{
+			"keywords/$draft":               nil,
+			w.rolePatch(mailbox.RoleSent):   true,
+			w.rolePatch(mailbox.RoleDrafts): nil,
+		}
+		for _, dir := range msg.CopyTo {
+			mbox, ok := w.dir2mbox[dir]
+			if ok && mbox != w.roles[mailbox.RoleSent] {
+				onSuccess[w.mboxPatch(mbox)] = true
+			}
+		}
 		// Create the submission
 		req.Invoke(&emailsubmission.Set{
 			Account: w.AccountId(),
@@ -77,11 +88,7 @@ func (w *JMAPWorker) handleStartSend(msg *types.StartSendingMessage) error {
 				},
 			},
 			OnSuccessUpdateEmail: map[jmap.ID]jmap.Patch{
-				"#sub": {
-					"keywords/$draft":               nil,
-					w.rolePatch(mailbox.RoleSent):   true,
-					w.rolePatch(mailbox.RoleDrafts): nil,
-				},
+				"#sub": onSuccess,
 			},
 		})
 
@@ -132,6 +139,11 @@ func (w *jmapSendWriter) Close() error {
 }
 
 func (w *JMAPWorker) getSenderIdentity(from *mail.Address) (jmap.ID, error) {
+	if len(w.identities) == 0 {
+		if err := w.GetIdentities(); err != nil {
+			return "", err
+		}
+	}
 	name, domain, _ := strings.Cut(from.Address, "@")
 	for _, ident := range w.identities {
 		n, d, _ := strings.Cut(ident.Email, "@")
