@@ -12,21 +12,21 @@ import (
 	"net/url"
 	"os"
 	"os/exec"
-	"path"
 	"path/filepath"
 	"strconv"
 	"strings"
 	"time"
 
 	"git.sr.ht/~rjarry/aerc/config"
+	"git.sr.ht/~rjarry/aerc/lib/log"
 	"git.sr.ht/~rjarry/aerc/lib/watchers"
 	"git.sr.ht/~rjarry/aerc/lib/xdg"
-	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/handlers"
 	"git.sr.ht/~rjarry/aerc/worker/lib"
 	notmuch "git.sr.ht/~rjarry/aerc/worker/notmuch/lib"
 	"git.sr.ht/~rjarry/aerc/worker/types"
+	"github.com/emersion/go-maildir"
 )
 
 func init() {
@@ -42,6 +42,7 @@ type worker struct {
 	currentQueryName    string
 	queryMapOrder       []string
 	nameQueryMap        map[string]string
+	dynamicNameQueryMap map[string]string
 	store               *lib.MaildirStore
 	maildirAccountPath  string
 	db                  *notmuch.DB
@@ -53,6 +54,7 @@ type worker struct {
 	headers             []string
 	headersExclude      []string
 	state               uint64
+	mfs                 types.MultiFileStrategy
 }
 
 // NewWorker creates a new notmuch worker with the provided worker.
@@ -70,6 +72,7 @@ func NewWorker(w *types.Worker) (types.Backend, error) {
 			Sort:   true,
 			Thread: true,
 		},
+		dynamicNameQueryMap: make(map[string]string),
 	}, nil
 }
 
@@ -178,6 +181,8 @@ func (w *worker) handleMessage(msg types.WorkerMessage) error {
 		return w.handleFlagMessages(msg)
 	case *types.AnsweredMessages:
 		return w.handleAnsweredMessages(msg)
+	case *types.ForwardedMessages:
+		return w.handleForwardedMessages(msg)
 	case *types.SearchDirectory:
 		return w.handleSearchDirectory(msg)
 	case *types.ModifyLabels:
@@ -240,6 +245,16 @@ func (w *worker) handleConfigure(msg *types.Configure) error {
 	w.headers = msg.Config.Headers
 	w.headersExclude = msg.Config.HeadersExclude
 
+	mfs := msg.Config.Params["multi-file-strategy"]
+	if mfs != "" {
+		w.mfs, ok = types.StrToStrategy[mfs]
+		if !ok {
+			return fmt.Errorf("invalid multi-file strategy %s", mfs)
+		}
+	} else {
+		w.mfs = types.Refuse
+	}
+
 	return nil
 }
 
@@ -250,10 +265,23 @@ func (w *worker) handleConnect(msg *types.Connect) error {
 	w.state = w.db.State()
 	// Watch all the files in the xapian folder for changes. We'll debounce
 	// changes, so catching multiple is ok
-	dbPath := path.Join(w.db.Path(), ".notmuch", "xapian")
-	err := w.watcher.Configure(dbPath)
+	var dbPath string
+	path := filepath.Join(w.db.Path(), ".notmuch", "xapian")
+	_, err := os.Stat(path)
 	if err != nil {
-		return err
+		if errors.Is(err, os.ErrNotExist) {
+			dbPath = filepath.Join(w.db.Path(), "xapian")
+		} else {
+			return fmt.Errorf("error locating notmuch db: %w", err)
+		}
+	} else {
+		dbPath = path
+	}
+
+	err = w.watcher.Configure(dbPath)
+	log.Tracef("Configuring watcher for path: %v", dbPath)
+	if err != nil {
+		return fmt.Errorf("error configuring watcher: %w", err)
 	}
 	return nil
 }
@@ -284,6 +312,17 @@ func (w *worker) handleListDirectories(msg *types.ListDirectories) error {
 			},
 		}, nil)
 	}
+
+	for name := range w.dynamicNameQueryMap {
+		w.w.PostMessage(&types.Directory{
+			Message: types.RespondTo(msg),
+			Dir: &models.Directory{
+				Name: name,
+				Role: models.QueryRole,
+			},
+		}, nil)
+	}
+
 	// Update dir counts when listing directories
 	err := w.updateDirCounts()
 	if err != nil {
@@ -318,14 +357,15 @@ func (w *worker) handleOpenDirectory(msg *types.OpenDirectory) error {
 	if msg.Context.Err() != nil {
 		return context.Canceled
 	}
-	w.w.Tracef("opening %s", msg.Directory)
+	w.w.Tracef("opening %s with query %s", msg.Directory, msg.Query)
 
-	var isDynamicFolder bool
+	var exists bool
 	q := ""
 	if w.store != nil {
 		folders, _ := w.store.FolderMap()
-		dir, ok := folders[msg.Directory]
-		if ok {
+		var dir maildir.Dir
+		dir, exists = folders[msg.Directory]
+		if exists {
 			folder := filepath.Join(w.maildirAccountPath, msg.Directory)
 			q = fmt.Sprintf("folder:%s", strconv.Quote(folder))
 			if err := w.processNewMaildirFiles(string(dir)); err != nil {
@@ -334,20 +374,27 @@ func (w *worker) handleOpenDirectory(msg *types.OpenDirectory) error {
 		}
 	}
 	if q == "" {
-		var ok bool
-		q, ok = w.nameQueryMap[msg.Directory]
-		if !ok {
-			q = msg.Directory
-			isDynamicFolder = true
-			w.w.PostMessage(&types.Directory{
-				Message: types.RespondTo(msg),
-				Dir: &models.Directory{
-					Name: q,
-					Role: models.QueryRole,
-				},
-			}, nil)
+		q, exists = w.nameQueryMap[msg.Directory]
+		if !exists {
+			q, exists = w.dynamicNameQueryMap[msg.Directory]
 		}
 	}
+	if !exists || msg.Force {
+		q = msg.Query
+		if q == "" {
+			q = msg.Directory
+		}
+		w.dynamicNameQueryMap[msg.Directory] = q
+		w.w.PostMessage(&types.Directory{
+			Message: types.RespondTo(msg),
+			Dir: &models.Directory{
+				Name: msg.Directory,
+				Role: models.QueryRole,
+			},
+		}, nil)
+	} else if msg.Query != "" && msg.Query != q {
+		return errors.New("cannot use existing folder name for new query")
+	}
 	w.query = q
 	w.currentQueryName = msg.Directory
 
@@ -355,7 +402,7 @@ func (w *worker) handleOpenDirectory(msg *types.OpenDirectory) error {
 		Info:    w.getDirectoryInfo(msg.Directory, w.query),
 		Message: types.RespondTo(msg),
 	}, nil)
-	if isDynamicFolder {
+	if !exists {
 		w.w.PostMessage(&types.DirectoryInfo{
 			Info:    w.getDirectoryInfo(msg.Directory, w.query),
 			Message: types.RespondTo(msg),
@@ -410,27 +457,21 @@ func (w *worker) handleFetchMessageHeaders(
 	return nil
 }
 
-func (w *worker) uidsFromQuery(ctx context.Context, query string) ([]uint32, error) {
+func (w *worker) uidsFromQuery(ctx context.Context, query string) ([]models.UID, error) {
 	msgIDs, err := w.db.MsgIDsFromQuery(ctx, query)
 	if err != nil {
 		return nil, err
 	}
-	var uids []uint32
+	var uids []models.UID
 	for _, id := range msgIDs {
-		uid := w.db.UidFromKey(id)
-		uids = append(uids, uid)
-
+		uids = append(uids, models.UID(id))
 	}
 	return uids, nil
 }
 
-func (w *worker) msgFromUid(uid uint32) (*Message, error) {
-	key, ok := w.db.KeyFromUid(uid)
-	if !ok {
-		return nil, fmt.Errorf("Invalid uid: %v", uid)
-	}
+func (w *worker) msgFromUid(uid models.UID) (*Message, error) {
 	msg := &Message{
-		key: key,
+		key: string(uid),
 		uid: uid,
 		db:  w.db,
 	}
@@ -511,6 +552,24 @@ func (w *worker) handleAnsweredMessages(msg *types.AnsweredMessages) error {
 	return nil
 }
 
+func (w *worker) handleForwardedMessages(msg *types.ForwardedMessages) error {
+	for _, uid := range msg.Uids {
+		m, err := w.msgFromUid(uid)
+		if err != nil {
+			w.w.Errorf("could not get message: %v", err)
+			w.err(msg, err)
+			continue
+		}
+		if err := m.MarkForwarded(msg.Forwarded); err != nil {
+			w.w.Errorf("could not mark message as forwarded: %v", err)
+			w.err(msg, err)
+			continue
+		}
+	}
+	w.done(msg)
+	return nil
+}
+
 func (w *worker) handleFlagMessages(msg *types.FlagMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
@@ -548,7 +607,7 @@ func (w *worker) handleModifyLabels(msg *types.ModifyLabels) error {
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			return fmt.Errorf("could not get message from uid %d: %w", uid, err)
+			return fmt.Errorf("could not get message from uid %s: %w", uid, err)
 		}
 		err = m.ModifyTags(msg.Add, msg.Remove)
 		if err != nil {
@@ -634,7 +693,7 @@ func (w *worker) emitDirectoryThreaded(parent types.WorkerMessage) error {
 	return nil
 }
 
-func (w *worker) emitMessageInfoError(msg types.WorkerMessage, uid uint32, err error) {
+func (w *worker) emitMessageInfoError(msg types.WorkerMessage, uid models.UID, err error) {
 	w.w.PostMessage(&types.MessageInfo{
 		Info: &models.MessageInfo{
 			Envelope: &models.Envelope{},
@@ -678,9 +737,9 @@ func (w *worker) emitLabelList() {
 	w.w.PostMessage(&types.LabelList{Labels: tags}, nil)
 }
 
-func (w *worker) sort(uids []uint32,
+func (w *worker) sort(uids []models.UID,
 	criteria []*types.SortCriterion,
-) ([]uint32, error) {
+) ([]models.UID, error) {
 	if len(criteria) == 0 {
 		return uids, nil
 	}
@@ -731,19 +790,14 @@ func (w *worker) handleDeleteMessages(msg *types.DeleteMessages) error {
 		return errUnsupported
 	}
 
-	var deleted []uint32
+	var deleted []models.UID
 
-	// With notmuch, two identical files can be referenced under
-	// the same index key, even if they exist in two different
-	// folders. So in order to remove the message from the right
-	// maildir folder we need to pass a hint to Remove() so it
-	// can purge the right file.
 	folders, _ := w.store.FolderMap()
-	path, ok := folders[w.currentQueryName]
-	if !ok {
-		w.err(msg, fmt.Errorf("Can only delete file from a maildir folder"))
-		w.done(msg)
-		return nil
+	curDir := folders[w.currentQueryName]
+
+	mfs := w.mfs
+	if msg.MultiFileStrategy != nil {
+		mfs = *msg.MultiFileStrategy
 	}
 
 	for _, uid := range msg.Uids {
@@ -753,7 +807,7 @@ func (w *worker) handleDeleteMessages(msg *types.DeleteMessages) error {
 			w.err(msg, err)
 			continue
 		}
-		if err := m.Remove(path); err != nil {
+		if err := m.Remove(curDir, mfs); err != nil {
 			w.w.Errorf("could not remove message: %v", err)
 			w.err(msg, err)
 			continue
@@ -765,8 +819,8 @@ func (w *worker) handleDeleteMessages(msg *types.DeleteMessages) error {
 			Message: types.RespondTo(msg),
 			Uids:    deleted,
 		}, nil)
+		w.done(msg)
 	}
-	w.done(msg)
 	return nil
 }
 
@@ -782,13 +836,20 @@ func (w *worker) handleCopyMessages(msg *types.CopyMessages) error {
 		return fmt.Errorf("Can only copy file to a maildir folder")
 	}
 
+	curDir := folders[w.currentQueryName]
+
+	mfs := w.mfs
+	if msg.MultiFileStrategy != nil {
+		mfs = *msg.MultiFileStrategy
+	}
+
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
 			w.w.Errorf("could not get message: %v", err)
 			return err
 		}
-		if err := m.Copy(dest); err != nil {
+		if err := m.Copy(curDir, dest, mfs); err != nil {
 			w.w.Errorf("could not copy message: %v", err)
 			return err
 		}
@@ -807,7 +868,7 @@ func (w *worker) handleMoveMessages(msg *types.MoveMessages) error {
 		return errUnsupported
 	}
 
-	var moved []uint32
+	var moved []models.UID
 
 	folders, _ := w.store.FolderMap()
 
@@ -817,6 +878,13 @@ func (w *worker) handleMoveMessages(msg *types.MoveMessages) error {
 		return fmt.Errorf("Can only move file to a maildir folder")
 	}
 
+	curDir := folders[w.currentQueryName]
+
+	mfs := w.mfs
+	if msg.MultiFileStrategy != nil {
+		mfs = *msg.MultiFileStrategy
+	}
+
 	var err error
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
@@ -824,22 +892,8 @@ func (w *worker) handleMoveMessages(msg *types.MoveMessages) error {
 			w.w.Errorf("could not get message: %v", err)
 			break
 		}
-		filenames, err := m.db.MsgFilenames(m.key)
-		if err != nil {
-			return err
-		}
-		// In the future, it'd be nice if we could overload move with
-		// the possibility to affect some or all of the files
-		// corresponding to a message.
-		if len(filenames) > 1 {
-			return fmt.Errorf("Cannot move: message %d has multiple files", m.uid)
-		}
-		source, key := parseFilename(filenames[0])
-		if key == "" {
-			return fmt.Errorf("failed to parse message filename: %s", filenames[0])
-		}
-		if err := m.Move(source, dest); err != nil {
-			w.w.Errorf("could not copy message: %v", err)
+		if err := m.Move(curDir, dest, mfs); err != nil {
+			w.w.Errorf("could not move message: %v", err)
 			break
 		}
 		moved = append(moved, uid)
@@ -921,6 +975,12 @@ func (w *worker) handleRemoveDirectory(msg *types.RemoveDirectory) error {
 		return errUnsupported
 	}
 
+	if _, ok := w.dynamicNameQueryMap[msg.Directory]; ok {
+		delete(w.dynamicNameQueryMap, msg.Directory)
+		w.done(msg)
+		return nil
+	}
+
 	if w.store == nil {
 		w.done(msg)
 		return nil
